#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# See the License for the specific language governing permissions
# and limitations under the License.
#
#
# Copyright 2011 Grigale Ltd. All rigths reserved.
# Use is sujbect to license terms.
#
import ctypes as C


KSTAT_TYPE_RAW = 0
KSTAT_TYPE_NAMED = 1
KSTAT_TYPE_INTR = 2
KSTAT_TYPE_IO = 3
KSTAT_TYPE_TIMER = 4

kstat_type_names = {
    KSTAT_TYPE_RAW: 'raw', KSTAT_TYPE_NAMED: 'named',
    KSTAT_TYPE_INTR: 'intr', KSTAT_TYPE_IO: 'io', KSTAT_TYPE_TIMER: 'timer'
    }

KSTAT_STRLEN = 31

hrtime_t = C.c_longlong
kid_t = C.c_int
kstat_string = C.c_char * KSTAT_STRLEN

class kstat(C.Structure):
    pass

kstat_p = C.POINTER(kstat)

kstat._fields_ = [
    ('ks_crtime', hrtime_t),
    ('ks_next', kstat_p),
    ('ks_kid', kid_t),
    ('ks_module', kstat_string),
    ('ks_resv', C.c_ubyte),
    ('ks_instance', C.c_int),
    ('ks_name', kstat_string),
    ('ks_type', C.c_ubyte),
    ('ks_class', kstat_string),
    ('ks_flags', C.c_ubyte),
    ('ks_data', C.c_void_p),
    ('ks_ndata', C.c_uint),
    ('ks_data_size', C.c_size_t),
    ('ks_snaptime', hrtime_t),

    ('ks_update', C.c_void_p),
    ('ks_private', C.c_void_p),
    ('ks_snapshot', C.c_void_p),
    ('ks_lock', C.c_void_p)
]


#/*
# * kstat_open() returns a pointer to a kstat_ctl_t.
# * This is used for subsequent libkstat operations.
# */
#typedef struct kstat_ctl {
#        kid_t   kc_chain_id;    /* current kstat chain ID       */
#        kstat_t *kc_chain;      /* pointer to kstat chain       */
#        int     kc_kd;          /* /dev/kstat descriptor        */
#} kstat_ctl_t;

class kstat_ctl(C.Structure):
    _fields_ = [
        ('kc_chain_id', kid_t),
        ('kc_chain', C.POINTER(kstat)),
        ('kc_kd', C.c_int)
    ]


#typedef struct kstat_named {
#        char    name[KSTAT_STRLEN];     /* name of counter */
#        uchar_t data_type;              /* data type */
#        union {
#                char            c[16];  /* enough for 128-bit ints */
#                int32_t         i32;
#                uint32_t        ui32;
#                struct {
#                        union {
#                                char            *ptr;   /* NULL-term string */
#                                char            __pad[8]; /* 64-bit padding */
#                        } addr;
#                        uint32_t        len;    /* # bytes for strlen + '\0' */
#                } str;
#/*
# * The int64_t and uint64_t types are not valid for a maximally conformant
# * 32-bit compilation environment (cc -Xc) using compilers prior to the
# * introduction of C99 conforming compiler (reference ISO/IEC 9899:1990).
# * In these cases, the visibility of i64 and ui64 is only permitted for
# * 64-bit compilation environments or 32-bit non-maximally conformant
# * C89 or C90 ANSI C compilation environments (cc -Xt and cc -Xa). In the
# * C99 ANSI C compilation environment, the long long type is supported.
# * The _INT64_TYPE is defined by the implementation (see sys/int_types.h).
# */
##if defined(_INT64_TYPE)
#                int64_t         i64;
#                uint64_t        ui64;
##endif
#                long            l;
#                ulong_t         ul;
#
#                /* These structure members are obsolete */
#
#                longlong_t      ll;
#                u_longlong_t    ull;
#                float           f;
#                double          d;
#        } value;                        /* value of counter */
#} kstat_named_t;

KSTAT_DATA_CHAR = 0
KSTAT_DATA_INT32 = 1
KSTAT_DATA_UINT32 = 2
KSTAT_DATA_INT64 = 3
KSTAT_DATA_UINT64 = 4
KSTAT_DATA_STRING = 9

class addr_union(C.Union):
    _fields_ = [
        ('ptr', C.c_char_p),
        ('__pad', C.c_char * 8),
    ]


class str_struct(C.Structure):
    _fields_ = [
        ('addr', addr_union),
        ('len', C.c_uint32),
    ]


class value_union(C.Union):
    _fields_ = [
        ('c', C.c_char * 16),
	('i32', C.c_int32),
	('ui32', C.c_uint32),
	('i64', C.c_int64),
	('ui64', C.c_uint64),
    ]


class kstat_named(C.Structure):
    _fields_ = [
        ('name', kstat_string),
        ('data_type', C.c_ubyte),
        ('value', value_union),
    ]


_libkstat = C.CDLL('libkstat.so.1')

kstat_ctl_p = C.POINTER(kstat_ctl)

kstat_open = _libkstat.kstat_open
kstat_open.argtypes = []
kstat_open.restype = kstat_ctl_p


kstat_close = _libkstat.kstat_close
kstat_close.argtypes = [kstat_ctl_p]


kstat_read = _libkstat.kstat_read
kstat_read.argtypes = [kstat_ctl_p, kstat_p, C.c_void_p]
kstat_read.restype = kid_t


kstat_write = _libkstat.kstat_write
kstat_write.argtypes = [kstat_ctl_p, kstat_p, C.c_void_p]
kstat_write.restype = kid_t


kstat_chain_update = _libkstat.kstat_chain_update
kstat_chain_update.argtypes = [kstat_ctl_p]
kstat_chain_update.restype = kid_t


kstat_lookup = _libkstat.kstat_lookup
kstat_lookup.argtypes = [kstat_ctl_p, C.c_char_p, C.c_int, C.c_char_p]
kstat_lookup.restype = kstat_p


kstat_data_lookup = _libkstat.kstat_data_lookup
kstat_data_lookup.argtypes = [kstat_p]
kstat_data_lookup.restype = C.c_void_p




class CompareUtil(object):

    @staticmethod
    def compareTo(l, r):
        if not l and not r:
            return 0
        elif not l:
            return -1
        elif not r:
            return 1

        l_ln = len(l)
        r_ln = len(r)
        for i in range(l_ln):
            if l[i] > r[i]:
                return 1
            if l[i] < r[i]:
                return -1
        return l_ln - r_ln



import threading
import Queue
import time
import whatap.agent.conf.configure as config
import whatap.agent.secure.security_master as secure
import whatap.pack.text_pack as text_pack
from whatap.util.date_util import DateUtil as dateutil
import whatap.util.hash_util as hash
import whatap.net.sender as sender
import whatap.net as net
from datetime import datetime

hash_cache = {}

class __DataText (threading.Thread):
    def __init__(self) :
        threading.Thread.__init__(self, name='__DataText')
        self.setDaemon(True)
        self.__queue = Queue.Queue()
        self.__hour = 0

    def queue(self, sendinfo):
        self.__queue.put(sendinfo)

    def run(self):
        
        while True:
            
            self.__run()
            self.__clearCache()
            time.sleep(10)

    def __clearCache(self):
        global hash_cache
        if datetime.now().hour != self.__hour:
            self.__hour = datetime.now().hour
            hash_cache.clear()

    def __run(self):
        rec = []
        try:
            while True:
                rec.append( self.__queue.get_nowait() )
        except Queue.Empty, e:
            pass

        if not rec:
            return
        sm= secure.GetSecurityMaster()
        p= text_pack.TextPack()
        p.pcode = sm.pcode
        p.oid = sm.oid
        p.time = dateutil.now()
        p.addTexts(rec)

        sender.Send(net.NET_SECURE_CYPHER, p)

dataText = __DataText()
dataText.start()

class TextKey :
    def __init__(self, div, hash):
        self.__div = div
        self.__hash = hash

    def Hash(self):
        return int(self.__.hash ^ int(self.__div<<32))

    def Equals(self, o):
        return self.__div == o.div and self.hash == o.hash

def sendText(div , text ):
    h = hash.HashStr(text)
    #dataText.queue( text_pack.TextRec(div= div, hash= h, text= text) )
    sendHashText(div, h, text)


def sendHashText(div , h , text ):
    global hash_cache
    if h not in hash_cache:
        dataText.queue(text_pack.TextRec(div=div, hash=h, text=text))
        hash_cache[h] = None





"""Convert "arbitrary" sound files to AIFF (Apple and SGI's audio format).

Input may be compressed.
Uncompressed file type may be AIFF, WAV, VOC, 8SVX, NeXT/Sun, and others.
An exception is raised if the file is not of a recognized type.
Returned filename is either the input filename or a temporary filename;
in the latter case the caller must ensure that it is removed.
Other temporary files used are removed by the function.
"""
from warnings import warnpy3k
warnpy3k("the toaiff module has been removed in Python 3.0", stacklevel=2)
del warnpy3k

import os
import tempfile
import pipes
import sndhdr

__all__ = ["error", "toaiff"]

table = {}

t = pipes.Template()
t.append('sox -t au - -t aiff -r 8000 -', '--')
table['au'] = t

# XXX The following is actually sub-optimal.
# XXX The HCOM sampling rate can be 22k, 22k/2, 22k/3 or 22k/4.
# XXX We must force the output sampling rate else the SGI won't play
# XXX files sampled at 5.5k or 7.333k; however this means that files
# XXX sampled at 11k are unnecessarily expanded.
# XXX Similar comments apply to some other file types.
t = pipes.Template()
t.append('sox -t hcom - -t aiff -r 22050 -', '--')
table['hcom'] = t

t = pipes.Template()
t.append('sox -t voc - -t aiff -r 11025 -', '--')
table['voc'] = t

t = pipes.Template()
t.append('sox -t wav - -t aiff -', '--')
table['wav'] = t

t = pipes.Template()
t.append('sox -t 8svx - -t aiff -r 16000 -', '--')
table['8svx'] = t

t = pipes.Template()
t.append('sox -t sndt - -t aiff -r 16000 -', '--')
table['sndt'] = t

t = pipes.Template()
t.append('sox -t sndr - -t aiff -r 16000 -', '--')
table['sndr'] = t

uncompress = pipes.Template()
uncompress.append('uncompress', '--')


class error(Exception):
    pass

def toaiff(filename):
    temps = []
    ret = None
    try:
        ret = _toaiff(filename, temps)
    finally:
        for temp in temps[:]:
            if temp != ret:
                try:
                    os.unlink(temp)
                except os.error:
                    pass
                temps.remove(temp)
    return ret

def _toaiff(filename, temps):
    if filename[-2:] == '.Z':
        (fd, fname) = tempfile.mkstemp()
        os.close(fd)
        temps.append(fname)
        sts = uncompress.copy(filename, fname)
        if sts:
            raise error, filename + ': uncompress failed'
    else:
        fname = filename
    try:
        ftype = sndhdr.whathdr(fname)
        if ftype:
            ftype = ftype[0] # All we're interested in
    except IOError, msg:
        if type(msg) == type(()) and len(msg) == 2 and \
                type(msg[0]) == type(0) and type(msg[1]) == type(''):
            msg = msg[1]
        if type(msg) != type(''):
            msg = repr(msg)
        raise error, filename + ': ' + msg
    if ftype == 'aiff':
        return fname
    if ftype is None or not ftype in table:
        raise error, '%s: unsupported audio file type %r' % (filename, ftype)
    (fd, temp) = tempfile.mkstemp()
    os.close(fd)
    temps.append(temp)
    sts = table[ftype].copy(fname, temp)
    if sts:
        raise error, filename + ': conversion to aiff failed'
    return temp



import os, time
import socket, fcntl, struct
import whatap.util.process_util as process_util
from whatap.util.hash_util import HashUtil as hash
from whatap.util.ip_util import IPUtil as ip_util
from whatap.pack.pack_enum import PackEnum
import whatap.pack.smnetperf_pack as smnetperf_pack
from StringIO import StringIO

class _NetPerfRaw:
    def __init__(self):
        self.deviceID = None
        self.readCount = None
        self.readByteCount = None
        self.readDroppedCount = None
        self.readErrorCount = None
        self.writeCount = None
        self.writeByteCount = None
        self.writeDroppedCount = None
        self.writeErrorCount = None
        self.timestamp = None

class _NetPerf:
    def __init__(self):
        self.desc = None
        self.ip = None
        self.hwAddr = None
        self.trafficIn = None
        self.trafficOut = None
        self.packetIn = None
        self.packetOut = None
        self.errorIn = None
        self.errorOut =  None
        self.droppedOut = None
        self.droppedIn = None
        self.linkUsageMax = None
        self.nicName = None

    def __str__(self):
        buf = StringIO()
        buf.write('\ndesc=')
        buf.write(self.desc)
        buf.write('\nip=')
        buf.write(self.ip)
        buf.write('\nhwAddr=')
        buf.write(self.hwAddr)
        buf.write('\ntrafficIn=')
        buf.write(self.trafficIn)
        buf.write('\ntrafficOut=')
        buf.write(self.trafficOut)
        buf.write('\npacketIn=')
        buf.write(self.packetIn)
        buf.write('\npacketOut=')
        buf.write(self.packetOut)
        buf.write('\nerrorIn=')
        buf.write(str(self.errorIn))
        buf.write('\nerrorOut=')
        buf.write(str(self.errorOut))
        buf.write('\ndroppedOut=')
        buf.write(str(self.droppedOut))
        buf.write('\ndroppedIn=')
        buf.write(str(self.droppedIn))

        return buf.getvalue()

def getIp(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    ip= fcntl.ioctl(
        s.fileno(),
        0x8915,  # SIOCGIFADDR
        struct.pack('256s', ifname[:15])
    )[20:24]

    info = fcntl.ioctl(s.fileno(), 0x8927,  struct.pack('256s', ifname[:15]))
    hwaddr = ':'.join(['%02x' % ord(char) for char in info[18:24]])

    return ip, hwaddr


oldNicTrafficMap = None
import psutil
def GetNicUtil():
    global oldNicTrafficMap
    kstatMap = {}

    kstatMap = psutil.net_io_counters(pernic=True)
    nicTrafficMap = {}

    ipinfo = {}
    ipmpList = set()
    ipmpMemberList = set()

    # IP / MAC via psutil (no subprocess)
    try:
        import socket as _socket
        _AF_LINK = getattr(psutil, 'AF_LINK', None)
        for iface, addr_list in psutil.net_if_addrs().items():
            if iface not in ipinfo:
                ipinfo[iface] = {}
            for addr in addr_list:
                if addr.family == _socket.AF_INET:
                    ipinfo[iface]['ip'] = addr.address
                elif _AF_LINK is not None and addr.family == _AF_LINK:
                    ipinfo[iface]['hwaddr'] = addr.address
    except Exception:
        pass

    # IPMP group membership
    try:
        ipmp_out = process_util.executeCommandShellWithTimeout('ipadm show-ipmp -p -o IPMPIF,MEMBER 2>/dev/null', timeout=10)
        for line in ipmp_out.split('\n'):
            line = line.strip()
            if ':' not in line:
                continue
            parts = line.split(':')
            if len(parts) < 2:
                continue
            group, member = parts[0].strip(), parts[1].strip()
            if group:
                ipmpList.add(group)
            if member and member != group:
                ipmpMemberList.add(member)
    except Exception:
        try:
            ifconfig = process_util.executeCommandShellWithTimeout('ifconfig -a', timeout=10)
            deviceId = None
            for l in ifconfig.split('\n'):
                if not l:
                    continue
                token = l.split()
                if not token:
                    continue
                if token[0].endswith(':'):
                    deviceId = token[0].split(':')[0]
                elif token[0] == 'groupname' and deviceId:
                    groupname = token[1]
                    ipmpList.add(groupname)
                    if groupname != deviceId:
                        ipmpMemberList.add(deviceId)
        except Exception:
            pass

    timestamp = time.time()
    traffics = []

    for deviceId, v in kstatMap.items():
        nicTrafficIoRaw = _NetPerfRaw()
        nicTrafficIoRaw.deviceId = deviceId
        nicTrafficIoRaw.readByteCount = int(v.bytes_recv)
        nicTrafficIoRaw.readCount = int(v.packets_recv)
        nicTrafficIoRaw.readErrorCount = int(v.errin)
        nicTrafficIoRaw.readDroppedCount = int(v.dropin)
        nicTrafficIoRaw.writeByteCount = int(v.bytes_sent)
        nicTrafficIoRaw.writeCount = int(v.packets_sent)
        nicTrafficIoRaw.writeErrorCount = int(v.errout)
        nicTrafficIoRaw.writeDroppedCount = int(v.dropout)
        nicTrafficIoRaw.timestamp = timestamp

        nicTrafficMap[deviceId] = nicTrafficIoRaw 
        if oldNicTrafficMap and deviceId in oldNicTrafficMap:
            oldNicTrafficIoRaw = oldNicTrafficMap[deviceId]
            timediff = nicTrafficIoRaw.timestamp - oldNicTrafficIoRaw.timestamp
            n = _NetPerf()
            n.desc = deviceId
            n.trafficIn = float(nicTrafficIoRaw.readByteCount - oldNicTrafficIoRaw.readByteCount) / timediff * 8
            n.trafficOut = float(nicTrafficIoRaw.writeByteCount - oldNicTrafficIoRaw.writeByteCount) / timediff * 8
            n.packetIn = float(nicTrafficIoRaw.readCount - oldNicTrafficIoRaw.readCount) / timediff 
            n.packetOut = float(nicTrafficIoRaw.writeCount - oldNicTrafficIoRaw.writeCount) / timediff
            n.droppedIn = float(nicTrafficIoRaw.readDroppedCount - oldNicTrafficIoRaw.readDroppedCount) / timediff
            n.droppedOut = float(nicTrafficIoRaw.writeDroppedCount - oldNicTrafficIoRaw.writeDroppedCount) / timediff
            n.errorIn = float(nicTrafficIoRaw.readErrorCount - oldNicTrafficIoRaw.readErrorCount) / timediff
            n.errorOut = float(nicTrafficIoRaw.writeErrorCount - oldNicTrafficIoRaw.writeErrorCount) / timediff
            if deviceId not in ipinfo or deviceId in ipmpMemberList or 'ip' not in ipinfo[deviceId]:
                continue
            n.ip = ip_util.toBytes(ipinfo[deviceId]['ip'])
            if 'hwaddr' not in ipinfo[deviceId]:
                n.hwAddr = ""
            else:
                n.hwAddr = ipinfo[deviceId]['hwaddr']


            traffics.append(n)
    oldNicTrafficMap = nicTrafficMap

    return traffics


_nicLinkSpeedCache = {}

def GetNicLinkSpeed(iface):
    if iface in _nicLinkSpeedCache:
        return _nicLinkSpeedCache[iface]
    speed_bps = None
    try:
        stats = psutil.net_if_stats()
        if iface in stats and stats[iface].speed and stats[iface].speed > 0:
            speed_bps = float(stats[iface].speed) * 1000000.0
    except Exception:
        speed_bps = None
    if speed_bps is None:
        try:
            output = process_util.executeCommandShellWithTimeout('dladm show-phys -p -o LINK,SPEED %s' % iface, timeout=10)
            for line in output.split('\n'):
                line = line.strip()
                if ':' in line:
                    parts = line.split(':')
                    if len(parts) >= 2:
                        val_str = parts[-1].strip()
                        if val_str and val_str.isdigit():
                            mbps = float(val_str)
                            if mbps > 0:
                                speed_bps = mbps * 1000000.0
                    break
        except Exception:
            speed_bps = None
    _nicLinkSpeedCache[iface] = speed_bps
    return speed_bps


_oldNetInfoPerNicMap = None

def GetNetInfo():
    global _oldNetInfoPerNicMap
    now = time.time()
    iocounters = psutil.net_io_counters(pernic=True)

    newMap = {}
    for deviceId, counter in iocounters.items():
        raw = _NetPerfRaw()
        raw.deviceId = deviceId
        raw.readByteCount = int(counter.bytes_recv)
        raw.readCount = int(counter.packets_recv)
        raw.readErrorCount = int(counter.errin)
        raw.readDroppedCount = int(counter.dropin)
        raw.writeByteCount = int(counter.bytes_sent)
        raw.writeCount = int(counter.packets_sent)
        raw.writeErrorCount = int(counter.errout)
        raw.writeDroppedCount = int(counter.dropout)
        raw.timestamp = now
        newMap[deviceId] = raw

    if not _oldNetInfoPerNicMap:
        _oldNetInfoPerNicMap = newMap
        return None

    total_rx_bps = 0.0
    total_tx_bps = 0.0
    total_rx_pps = 0.0
    total_tx_pps = 0.0
    max_usage = -1.0
    max_nic_name = None

    for deviceId, raw in newMap.items():
        if deviceId not in _oldNetInfoPerNicMap:
            continue
        old = _oldNetInfoPerNicMap[deviceId]
        timediff = raw.timestamp - old.timestamp
        if timediff <= 0:
            continue
        rx_bps = float(raw.readByteCount - old.readByteCount) / timediff * 8
        tx_bps = float(raw.writeByteCount - old.writeByteCount) / timediff * 8
        rx_pps = float(raw.readCount - old.readCount) / timediff
        tx_pps = float(raw.writeCount - old.writeCount) / timediff
        total_rx_bps += rx_bps
        total_tx_bps += tx_bps
        total_rx_pps += rx_pps
        total_tx_pps += tx_pps

        if not deviceId.startswith('lo'):
            link_speed = GetNicLinkSpeed(deviceId)
            if link_speed and link_speed > 0:
                usage = (rx_bps + tx_bps) / link_speed * 100.0
                if usage > max_usage:
                    max_usage = usage
                    max_nic_name = deviceId

    _oldNetInfoPerNicMap = newMap

    n = _NetPerf()
    n.trafficIn = total_rx_bps
    n.trafficOut = total_tx_bps
    n.packetIn = total_rx_pps
    n.packetOut = total_tx_pps
    if max_usage >= 0:
        n.linkUsageMax = min(max_usage, 100.0)
        n.nicName = max_nic_name
    return n


def test():
    print 'test'
    print GetNicUtil()
    time.sleep(5)
    for netperf in GetNicUtil():
        print netperf

if __name__ == '__main__':
    test()



import threading, time, os, re

import whatap.agent.conf.configure as config
from StringIO import StringIO
from datetime import datetime
import glob
import whatap.util.logging_util as logging_util
from whatap.util.date_util import DateUtil


start = False
lock = threading.Lock()

def initFileLogWatcher():
    global start
    if not start :
        lock.acquire()
        if not start :
            start = True
            t = threading.Thread(target=checkFileLogConfig)
            t.daemon = True
            t.start()
            t = threading.Thread(target=startTail)
            t.daemon = True
            t.start()
        lock.release()

def parseLogConfig(logConfig ):
    words = logConfig.split(",")
    #print 'parseLogConfig step -1', logConfig
    word_count = len(words)
    if word_count < 3:
        return None, None, None
    severity = int(words[0])
    logfile = words[1]
    regexPattern = words[2]
    regexExcludePattern = None
    duration = None
    repeat = None
    if word_count > 3:
        duration = int(words[3])
    if word_count > 4:
        repeat = int(words[4])
    if word_count > 5:
        regexExcludePattern = words[5]

    return int(severity), logfile, regexPattern, duration, repeat, regexExcludePattern

class LogCondition(object):
    def __init__(self, regexPattern = None, regexExcludePattern = None, severity = None, triggerId=None, duration = None, repeat = None):
        self.regexPattern = regexPattern
        self.regexExcludePattern = regexExcludePattern
        self.severity = severity
        self.triggerId = triggerId
        self.duration = duration
        self.repeat = repeat

    def clone(self):
        return LogCondition(regexPattern = self.regexPattern, regexExcludePattern = self.regexExcludePattern, severity = self.severity, triggerId=self.triggerId, duration = self.duration, repeat = self.repeat)

    def  __str__(self):
        buf = StringIO()
        buf.write('regexPattern=')
        buf.write(self.regexPattern)
        buf.write('regexExcludePattern=')
        buf.write(self.regexExcludePattern)
        buf.write('severity=')
        buf.write(self.severity)
        buf.write('triggerId=')
        buf.write(self.triggerId)
        buf.write('duration=')
        buf.write(self.duration)
        buf.write('repeat=')
        buf.write(self.repeat)

class LogStatus:
    def __init__(self, filePos = 0, filePath = None, realFilePath = None, logConditions = None, fileInfo = None, watchCount = None):
        self.filePos  = filePos
        self.filePath = filePath
        self.realFilePath = realFilePath
        self.logConditions = logConditions
        self.watchCount  = watchCount
        self.fileInfo = fileInfo
        self.realFileCheckIdx = 0

    def clone(self):
        clonedLogConditions = {}
        if self.logConditions:
            clonedLogConditions = { k : v.clone() for k, v in self.logConditions.items()}

        l = LogStatus(filePos = self.filePos, filePath = self.filePath, realFilePath = self.realFilePath, logConditions= clonedLogConditions, fileInfo = self.fileInfo, watchCount = self.watchCount)
        return l

    def __str__(self):
        buf = StringIO()
        buf.write('filePos=')
        buf.write(self.filePos)
        buf.write('\nfilePath=')
        buf.write(self.filePath)
        buf.write('\nrealFilePath=')
        buf.write(self.realFilePath)
        if self.logConditions:
            for k, v in self.logConditions.items():
                buf.write(k)
                buf.write('=LogCondition')
                buf.write(v)

        buf.write('\nwatchCount=')
        buf.write(self.watchCount)
        buf.write('\nfileInfo=')
        buf.write(self.fileInfo)

        return buf.getvalue()

class LogEventCount(object):
    def __init__(self, count = 0, firstOccured = None):
        self.count = count
        self.firstOccured = firstOccured

    def __str__(self):
        buf = StringIO()
        buf.write("count=")
        buf.write(self.count)
        buf.write("firstOccured=")
        buf.write(self.firstOccured)

        return buf.getvalue()

logStatus = {}
logMatches = {}
logCounter = {}
loglock = threading.Lock()

def addLogMatch(logfile , reallogfile , logline, logCondition ):
    if logfile not in logMatches:
        logMatches[logfile] = []
    logMatches[logfile].append((reallogfile, logline, logCondition))

class LogEvent:
    def __init__(self, filePath = None, logContent = None, severity = None, triggerId = None, target=None, keyword = None):
        self.filePath = filePath
        self.logContent = logContent
        self.severity = severity
        self.triggerId = triggerId
        self.target = target
        self.keyword = keyword

    def __str__(self):
        buf = StringIO()
        buf.write("filePath=")
        buf.write(self.filePath)
        buf.write("\nlogContent=")
        buf.write(self.logContent)
        buf.write("\nseverity=")
        buf.write(self.severity)
        buf.write("\ntriggerId=")
        buf.write(self.triggerId)
        buf.write("\ntarget=")
        buf.write(self.target)
        buf.write("\nkeyword=")
        buf.write(self.keyword)

        return buf.getvalue()

def getNewLogEvents():
    global loglock
    global logMatches
    global logStatus
    loglock.acquire()
    try:
        logevents=[]
        #print('getNewLogEvents step -1')
        for logfile, logmatches in logMatches.items():
            #print('getNewLogEvents step -2')
            for (reallogfile, logline, logCondition) in logmatches:
                #print('getNewLogEvents step -3')
                if logCondition.regexExcludePattern:
                    target = logCondition.regexPattern + " e:" + logCondition.regexExcludePattern
                else:
                    target = logCondition.regexPattern
                #print('getNewLogEvents step -4')
                logevents.append(LogEvent(filePath= reallogfile, logContent= logline, severity= logCondition.severity,\
                                          triggerId= logCondition.triggerId, target= target, keyword = logCondition.regexPattern))

            del logMatches[logfile]
        #print('getNewLogEvents step -5', logevents)
        return logevents
    finally:
        loglock.release()

def startTail():
    logStatusClone = {}
    i =0
    while True:
        i += 1
        t = datetime.now()

        for v in logStatus.values():
            evaluatedPath = t.strftime(v.filePath)
            #print('startTail step -2')
            matches = glob.glob(evaluatedPath)
            for evaluatedPath in matches:
                if evaluatedPath not in logStatusClone:
                    vv = v.clone()
                    vv.realFilePath = evaluatedPath
                    logStatusClone[evaluatedPath] = vv
                else:
                    logStatusClone[evaluatedPath].logConditions = v.clone().logConditions
                logStatusClone[evaluatedPath].realFileCheckIdx = i
        #print('startTail step -3')
        obsoleted = []
        for k, v in logStatusClone.items():
            if v.realFileCheckIdx != i:
                obsoleted.append(k)
        for k in obsoleted:
            del logStatusClone[k]
        #print('startTail step -4')
        for v in logStatusClone.values():
            logfile = v.realFilePath

            try:
                f = open(logfile)
            except IOError, e:
                continue
            #print('startTail step -5')
            fi = os.stat(logfile)

            if i == 0 or not v.fileInfo:
                v.fileInfo = fi
                f.seek(0, 2)
                v.filePos = f.tell()
                f.close()
                continue
            #print('startTail step -6')
            if fi.st_ino == v.fileInfo.st_ino:
                v.fileInfo = fi
                if v.filePos == fi.st_size:
                    f.close()
                    continue
            else:
                v.fileInfo = fi
                v.filePos = 0
            #print('startTail step -7')
            f.seek(0, 2)
            pos = f.tell()

            if v.filePos < pos:
                #print('startTail step -8')
                nbyteleft = pos - v.filePos
                f.seek(v.filePos, 0)
                pos = watchInc(f, v, nbyteleft)
                v.filePos = pos

            elif v.filePos > pos:
                nbyteleft = pos
                f.seek(0, 0)
                #print('startTail step -9')
                v.filePos = watchInc(f, v, nbyteleft)

            if f:
                f.close()
        #print('startTail step -10')
        time.sleep(1)
    


def watchInc(f , logrule , limit):
    lineLimit = config.GetConfig().LogLineLimit
    lineCount = 0
    while lineCount < lineLimit:
        lineCount+=1
        curr_position = f.tell()
        #print 'watchInc step -1'
        line = f.readline()
        if not line:
            break
        #print 'watchInc step -1', line
        for logCondition in logrule.logConditions.values():
            regexpatterncompile = None
            try:
                if logCondition.regexPattern:
                    regexpatterncompile = re.compile(logCondition.regexPattern)
            except Exception ,e:
                logging_util.debugStack(e)
                continue
            regexexcludepatterncompile = None
            try:
                if logCondition.regexExcludePattern:
                    regexexcludepatterncompile = re.compile(logCondition.regexExcludePattern)
            except Exception ,e:
                logging_util.debugStack(e)
                continue

            if not line:
                f.seek(curr_position,0)
                break
            else:
                #print('watchInc step -2')
                if not regexpatterncompile or len(regexpatterncompile.findall(line)) > 0:
                    if regexexcludepatterncompile and len(regexexcludepatterncompile.findall(line)) > 0:
                        time.sleep(0.001)
                        continue
                    #print('watchInc step -4')
                    logCounterKey = f.name + "\\" + logCondition.regexPattern
                    now = DateUtil.now()
                    logEventCount = logCounter.get(logCounterKey)
                    if logEventCount:
                        logging_util.debug("duration Check ", logCounterKey, " time from first occured:",
                                    now - logEventCount.firstOccured, " duration:", (logCondition.duration * 1000))
                    if not logEventCount or now-logEventCount.firstOccured > (logCondition.duration * 1000):
                        logEventCount = LogEventCount(firstOccured= now)
                        logCounter[logCounterKey] = logEventCount
                        logging_util.debug("log match created ", logCounterKey)
                    #print ('watchInc step -5')
                    logEventCount.count +=1
                    logCounter[logCounterKey] = logEventCount
                    logging_util.debug("log ", logCounterKey, " ", logEventCount.count, "/", logCondition.repeat)
                    logging_util.debug("log repeat condition ", logEventCount.count == logCondition.repeat,
                                    now - logEventCount.firstOccured <= (logCondition.duration * 1000))
                    if logEventCount.count == logCondition.repeat and \
                        now - logEventCount.firstOccured <= (logCondition.duration * 1000):
                        addLogMatch(logrule.filePath, logrule.realFilePath, line, logCondition)

        time.sleep(0.001)

    pos = f.tell()

    return pos

logStatus = {}
logMatches = {}


def __checkFileLogConfig():
    global logStatus
    global logMatches
    logStatusClone = {}
    for k, v in logStatus.items():
        logStatusClone[k] = v.clone()
    logfiles = config.GetConfig().searchKey(config.LogFileWatch)
    for key, logfilerule in logfiles.items():
        #print("checkFileLogConfig ", key, logfilerule)
        if len(key.split('.')) < 4:
            continue
        triggerId, triggerHash = key.split(".")[2:4]
        if logfilerule:
            severity, logfile, regexpattern, duration, repeat, regexexcludepattern = parseLogConfig(logfilerule)
            if logfile in logStatusClone:
                logConditions = logStatusClone[logfile].logConditions
                if triggerHash in logConditions:
                    del logConditions[triggerHash]
                logStatusClone[logfile].logConditions = logConditions
                if not logStatusClone[logfile].logConditions:
                    del logStatusClone[logfile ]
            logCondition = LogCondition(regexPattern= regexpattern,regexExcludePattern=  regexexcludepattern,\
                                   severity= severity,triggerId= triggerId, duration= duration, repeat= repeat)
            if logfile not in logStatus:
                logConditions = {}
                logStatus[logfile] = LogStatus(filePos= -1, filePath= logfile, logConditions= logConditions)

            logStatus[logfile].logConditions[triggerHash] = logCondition

    for logfile in logStatusClone.keys() :
        if logStatusClone[logfile].logConditions:
            for triggerHash in logStatusClone[logfile].logConditions :
                del logStatus[logfile].logConditions[triggerHash]
        else:
            del logStatus[logfile]

def checkFileLogConfig():
    global loglock
    while True:
        try:
            loglock.acquire()
            __checkFileLogConfig()
        except Exception, e:
            logging_util.debugStack(e)
        finally:
            loglock.release()
        time.sleep(5)


def test():
    initFileLogWatcher()
    while True:
        print getNewLogEvents()
        time.sleep(1)

if __name__ == '__main__':
    test()



import unittest, operator, copy, pickle, random
from test import test_support

test_support.import_module("sets", deprecated=True)
from sets import Set, ImmutableSet

empty_set = Set()

#==============================================================================

class TestBasicOps(unittest.TestCase):

    def test_repr(self):
        if self.repr is not None:
            self.assertEqual(repr(self.set), self.repr)

    def test_length(self):
        self.assertEqual(len(self.set), self.length)

    def test_self_equality(self):
        self.assertEqual(self.set, self.set)

    def test_equivalent_equality(self):
        self.assertEqual(self.set, self.dup)

    def test_copy(self):
        self.assertEqual(self.set.copy(), self.dup)

    def test_self_union(self):
        result = self.set | self.set
        self.assertEqual(result, self.dup)

    def test_empty_union(self):
        result = self.set | empty_set
        self.assertEqual(result, self.dup)

    def test_union_empty(self):
        result = empty_set | self.set
        self.assertEqual(result, self.dup)

    def test_self_intersection(self):
        result = self.set & self.set
        self.assertEqual(result, self.dup)

    def test_empty_intersection(self):
        result = self.set & empty_set
        self.assertEqual(result, empty_set)

    def test_intersection_empty(self):
        result = empty_set & self.set
        self.assertEqual(result, empty_set)

    def test_self_symmetric_difference(self):
        result = self.set ^ self.set
        self.assertEqual(result, empty_set)

    def checkempty_symmetric_difference(self):
        result = self.set ^ empty_set
        self.assertEqual(result, self.set)

    def test_self_difference(self):
        result = self.set - self.set
        self.assertEqual(result, empty_set)

    def test_empty_difference(self):
        result = self.set - empty_set
        self.assertEqual(result, self.dup)

    def test_empty_difference_rev(self):
        result = empty_set - self.set
        self.assertEqual(result, empty_set)

    def test_iteration(self):
        for v in self.set:
            self.assertIn(v, self.values)

    def test_pickling(self):
        for proto in range(pickle.HIGHEST_PROTOCOL + 1):
            p = pickle.dumps(self.set, proto)
            copy = pickle.loads(p)
            self.assertEqual(self.set, copy,
                             "%s != %s" % (self.set, copy))

#------------------------------------------------------------------------------

class TestBasicOpsEmpty(TestBasicOps):
    def setUp(self):
        self.case   = "empty set"
        self.values = []
        self.set    = Set(self.values)
        self.dup    = Set(self.values)
        self.length = 0
        self.repr   = "Set([])"

#------------------------------------------------------------------------------

class TestBasicOpsSingleton(TestBasicOps):
    def setUp(self):
        self.case   = "unit set (number)"
        self.values = [3]
        self.set    = Set(self.values)
        self.dup    = Set(self.values)
        self.length = 1
        self.repr   = "Set([3])"

    def test_in(self):
        self.assertTrue(3 in self.set)

    def test_not_in(self):
        self.assertTrue(2 not in self.set)

#------------------------------------------------------------------------------

class TestBasicOpsTuple(TestBasicOps):
    def setUp(self):
        self.case   = "unit set (tuple)"
        self.values = [(0, "zero")]
        self.set    = Set(self.values)
        self.dup    = Set(self.values)
        self.length = 1
        self.repr   = "Set([(0, 'zero')])"

    def test_in(self):
        self.assertTrue((0, "zero") in self.set)

    def test_not_in(self):
        self.assertTrue(9 not in self.set)

#------------------------------------------------------------------------------

class TestBasicOpsTriple(TestBasicOps):
    def setUp(self):
        self.case   = "triple set"
        self.values = [0, "zero", operator.add]
        self.set    = Set(self.values)
        self.dup    = Set(self.values)
        self.length = 3
        self.repr   = None

#==============================================================================

def baditer():
    raise TypeError
    yield True

def gooditer():
    yield True

class TestExceptionPropagation(unittest.TestCase):
    """SF 628246:  Set constructor should not trap iterator TypeErrors"""

    def test_instanceWithException(self):
        self.assertRaises(TypeError, Set, baditer())

    def test_instancesWithoutException(self):
        # All of these iterables should load without exception.
        Set([1,2,3])
        Set((1,2,3))
        Set({'one':1, 'two':2, 'three':3})
        Set(xrange(3))
        Set('abc')
        Set(gooditer())

#==============================================================================

class TestSetOfSets(unittest.TestCase):
    def test_constructor(self):
        inner = Set([1])
        outer = Set([inner])
        element = outer.pop()
        self.assertEqual(type(element), ImmutableSet)
        outer.add(inner)        # Rebuild set of sets with .add method
        outer.remove(inner)
        self.assertEqual(outer, Set())   # Verify that remove worked
        outer.discard(inner)    # Absence of KeyError indicates working fine

#==============================================================================

class TestBinaryOps(unittest.TestCase):
    def setUp(self):
        self.set = Set((2, 4, 6))

    def test_eq(self):              # SF bug 643115
        self.assertEqual(self.set, Set({2:1,4:3,6:5}))

    def test_union_subset(self):
        result = self.set | Set([2])
        self.assertEqual(result, Set((2, 4, 6)))

    def test_union_superset(self):
        result = self.set | Set([2, 4, 6, 8])
        self.assertEqual(result, Set([2, 4, 6, 8]))

    def test_union_overlap(self):
        result = self.set | Set([3, 4, 5])
        self.assertEqual(result, Set([2, 3, 4, 5, 6]))

    def test_union_non_overlap(self):
        result = self.set | Set([8])
        self.assertEqual(result, Set([2, 4, 6, 8]))

    def test_intersection_subset(self):
        result = self.set & Set((2, 4))
        self.assertEqual(result, Set((2, 4)))

    def test_intersection_superset(self):
        result = self.set & Set([2, 4, 6, 8])
        self.assertEqual(result, Set([2, 4, 6]))

    def test_intersection_overlap(self):
        result = self.set & Set([3, 4, 5])
        self.assertEqual(result, Set([4]))

    def test_intersection_non_overlap(self):
        result = self.set & Set([8])
        self.assertEqual(result, empty_set)

    def test_sym_difference_subset(self):
        result = self.set ^ Set((2, 4))
        self.assertEqual(result, Set([6]))

    def test_sym_difference_superset(self):
        result = self.set ^ Set((2, 4, 6, 8))
        self.assertEqual(result, Set([8]))

    def test_sym_difference_overlap(self):
        result = self.set ^ Set((3, 4, 5))
        self.assertEqual(result, Set([2, 3, 5, 6]))

    def test_sym_difference_non_overlap(self):
        result = self.set ^ Set([8])
        self.assertEqual(result, Set([2, 4, 6, 8]))

    def test_cmp(self):
        a, b = Set('a'), Set('b')
        self.assertRaises(TypeError, cmp, a, b)

        # You can view this as a buglet:  cmp(a, a) does not raise TypeError,
        # because __eq__ is tried before __cmp__, and a.__eq__(a) returns True,
        # which Python thinks is good enough to synthesize a cmp() result
        # without calling __cmp__.
        self.assertEqual(cmp(a, a), 0)

        self.assertRaises(TypeError, cmp, a, 12)
        self.assertRaises(TypeError, cmp, "abc", a)

    def test_inplace_on_self(self):
        t = self.set.copy()
        t |= t
        self.assertEqual(t, self.set)
        t &= t
        self.assertEqual(t, self.set)
        t -= t
        self.assertEqual(len(t), 0)
        t = self.set.copy()
        t ^= t
        self.assertEqual(len(t), 0)


#==============================================================================

class TestUpdateOps(unittest.TestCase):
    def setUp(self):
        self.set = Set((2, 4, 6))

    def test_union_subset(self):
        self.set |= Set([2])
        self.assertEqual(self.set, Set((2, 4, 6)))

    def test_union_superset(self):
        self.set |= Set([2, 4, 6, 8])
        self.assertEqual(self.set, Set([2, 4, 6, 8]))

    def test_union_overlap(self):
        self.set |= Set([3, 4, 5])
        self.assertEqual(self.set, Set([2, 3, 4, 5, 6]))

    def test_union_non_overlap(self):
        self.set |= Set([8])
        self.assertEqual(self.set, Set([2, 4, 6, 8]))

    def test_union_method_call(self):
        self.set.union_update(Set([3, 4, 5]))
        self.assertEqual(self.set, Set([2, 3, 4, 5, 6]))

    def test_intersection_subset(self):
        self.set &= Set((2, 4))
        self.assertEqual(self.set, Set((2, 4)))

    def test_intersection_superset(self):
        self.set &= Set([2, 4, 6, 8])
        self.assertEqual(self.set, Set([2, 4, 6]))

    def test_intersection_overlap(self):
        self.set &= Set([3, 4, 5])
        self.assertEqual(self.set, Set([4]))

    def test_intersection_non_overlap(self):
        self.set &= Set([8])
        self.assertEqual(self.set, empty_set)

    def test_intersection_method_call(self):
        self.set.intersection_update(Set([3, 4, 5]))
        self.assertEqual(self.set, Set([4]))

    def test_sym_difference_subset(self):
        self.set ^= Set((2, 4))
        self.assertEqual(self.set, Set([6]))

    def test_sym_difference_superset(self):
        self.set ^= Set((2, 4, 6, 8))
        self.assertEqual(self.set, Set([8]))

    def test_sym_difference_overlap(self):
        self.set ^= Set((3, 4, 5))
        self.assertEqual(self.set, Set([2, 3, 5, 6]))

    def test_sym_difference_non_overlap(self):
        self.set ^= Set([8])
        self.assertEqual(self.set, Set([2, 4, 6, 8]))

    def test_sym_difference_method_call(self):
        self.set.symmetric_difference_update(Set([3, 4, 5]))
        self.assertEqual(self.set, Set([2, 3, 5, 6]))

    def test_difference_subset(self):
        self.set -= Set((2, 4))
        self.assertEqual(self.set, Set([6]))

    def test_difference_superset(self):
        self.set -= Set((2, 4, 6, 8))
        self.assertEqual(self.set, Set([]))

    def test_difference_overlap(self):
        self.set -= Set((3, 4, 5))
        self.assertEqual(self.set, Set([2, 6]))

    def test_difference_non_overlap(self):
        self.set -= Set([8])
        self.assertEqual(self.set, Set([2, 4, 6]))

    def test_difference_method_call(self):
        self.set.difference_update(Set([3, 4, 5]))
        self.assertEqual(self.set, Set([2, 6]))

#==============================================================================

class TestMutate(unittest.TestCase):
    def setUp(self):
        self.values = ["a", "b", "c"]
        self.set = Set(self.values)

    def test_add_present(self):
        self.set.add("c")
        self.assertEqual(self.set, Set("abc"))

    def test_add_absent(self):
        self.set.add("d")
        self.assertEqual(self.set, Set("abcd"))

    def test_add_until_full(self):
        tmp = Set()
        expected_len = 0
        for v in self.values:
            tmp.add(v)
            expected_len += 1
            self.assertEqual(len(tmp), expected_len)
        self.assertEqual(tmp, self.set)

    def test_remove_present(self):
        self.set.remove("b")
        self.assertEqual(self.set, Set("ac"))

    def test_remove_absent(self):
        try:
            self.set.remove("d")
            self.fail("Removing missing element should have raised LookupError")
        except LookupError:
            pass

    def test_remove_until_empty(self):
        expected_len = len(self.set)
        for v in self.values:
            self.set.remove(v)
            expected_len -= 1
            self.assertEqual(len(self.set), expected_len)

    def test_discard_present(self):
        self.set.discard("c")
        self.assertEqual(self.set, Set("ab"))

    def test_discard_absent(self):
        self.set.discard("d")
        self.assertEqual(self.set, Set("abc"))

    def test_clear(self):
        self.set.clear()
        self.assertEqual(len(self.set), 0)

    def test_pop(self):
        popped = {}
        while self.set:
            popped[self.set.pop()] = None
        self.assertEqual(len(popped), len(self.values))
        for v in self.values:
            self.assertIn(v, popped)

    def test_update_empty_tuple(self):
        self.set.union_update(())
        self.assertEqual(self.set, Set(self.values))

    def test_update_unit_tuple_overlap(self):
        self.set.union_update(("a",))
        self.assertEqual(self.set, Set(self.values))

    def test_update_unit_tuple_non_overlap(self):
        self.set.union_update(("a", "z"))
        self.assertEqual(self.set, Set(self.values + ["z"]))

#==============================================================================

class TestSubsets(unittest.TestCase):

    case2method = {"<=": "issubset",
                   ">=": "issuperset",
                  }

    reverse = {"==": "==",
               "!=": "!=",
               "<":  ">",
               ">":  "<",
               "<=": ">=",
               ">=": "<=",
              }

    def test_issubset(self):
        x = self.left
        y = self.right
        for case in "!=", "==", "<", "<=", ">", ">=":
            expected = case in self.cases
            # Test the binary infix spelling.
            result = eval("x" + case + "y", locals())
            self.assertEqual(result, expected)
            # Test the "friendly" method-name spelling, if one exists.
            if case in TestSubsets.case2method:
                method = getattr(x, TestSubsets.case2method[case])
                result = method(y)
                self.assertEqual(result, expected)

            # Now do the same for the operands reversed.
            rcase = TestSubsets.reverse[case]
            result = eval("y" + rcase + "x", locals())
            self.assertEqual(result, expected)
            if rcase in TestSubsets.case2method:
                method = getattr(y, TestSubsets.case2method[rcase])
                result = method(x)
                self.assertEqual(result, expected)
#------------------------------------------------------------------------------

class TestSubsetEqualEmpty(TestSubsets):
    left  = Set()
    right = Set()
    name  = "both empty"
    cases = "==", "<=", ">="

#------------------------------------------------------------------------------

class TestSubsetEqualNonEmpty(TestSubsets):
    left  = Set([1, 2])
    right = Set([1, 2])
    name  = "equal pair"
    cases = "==", "<=", ">="

#------------------------------------------------------------------------------

class TestSubsetEmptyNonEmpty(TestSubsets):
    left  = Set()
    right = Set([1, 2])
    name  = "one empty, one non-empty"
    cases = "!=", "<", "<="

#------------------------------------------------------------------------------

class TestSubsetPartial(TestSubsets):
    left  = Set([1])
    right = Set([1, 2])
    name  = "one a non-empty proper subset of other"
    cases = "!=", "<", "<="

#------------------------------------------------------------------------------

class TestSubsetNonOverlap(TestSubsets):
    left  = Set([1])
    right = Set([2])
    name  = "neither empty, neither contains"
    cases = "!="

#==============================================================================

class TestOnlySetsInBinaryOps(unittest.TestCase):

    def test_eq_ne(self):
        # Unlike the others, this is testing that == and != *are* allowed.
        self.assertEqual(self.other == self.set, False)
        self.assertEqual(self.set == self.other, False)
        self.assertEqual(self.other != self.set, True)
        self.assertEqual(self.set != self.other, True)

    def test_ge_gt_le_lt(self):
        self.assertRaises(TypeError, lambda: self.set < self.other)
        self.assertRaises(TypeError, lambda: self.set <= self.other)
        self.assertRaises(TypeError, lambda: self.set > self.other)
        self.assertRaises(TypeError, lambda: self.set >= self.other)

        self.assertRaises(TypeError, lambda: self.other < self.set)
        self.assertRaises(TypeError, lambda: self.other <= self.set)
        self.assertRaises(TypeError, lambda: self.other > self.set)
        self.assertRaises(TypeError, lambda: self.other >= self.set)

    def test_union_update_operator(self):
        try:
            self.set |= self.other
        except TypeError:
            pass
        else:
            self.fail("expected TypeError")

    def test_union_update(self):
        if self.otherIsIterable:
            self.set.union_update(self.other)
        else:
            self.assertRaises(TypeError, self.set.union_update, self.other)

    def test_union(self):
        self.assertRaises(TypeError, lambda: self.set | self.other)
        self.assertRaises(TypeError, lambda: self.other | self.set)
        if self.otherIsIterable:
            self.set.union(self.other)
        else:
            self.assertRaises(TypeError, self.set.union, self.other)

    def test_intersection_update_operator(self):
        try:
            self.set &= self.other
        except TypeError:
            pass
        else:
            self.fail("expected TypeError")

    def test_intersection_update(self):
        if self.otherIsIterable:
            self.set.intersection_update(self.other)
        else:
            self.assertRaises(TypeError,
                              self.set.intersection_update,
                              self.other)

    def test_intersection(self):
        self.assertRaises(TypeError, lambda: self.set & self.other)
        self.assertRaises(TypeError, lambda: self.other & self.set)
        if self.otherIsIterable:
            self.set.intersection(self.other)
        else:
            self.assertRaises(TypeError, self.set.intersection, self.other)

    def test_sym_difference_update_operator(self):
        try:
            self.set ^= self.other
        except TypeError:
            pass
        else:
            self.fail("expected TypeError")

    def test_sym_difference_update(self):
        if self.otherIsIterable:
            self.set.symmetric_difference_update(self.other)
        else:
            self.assertRaises(TypeError,
                              self.set.symmetric_difference_update,
                              self.other)

    def test_sym_difference(self):
        self.assertRaises(TypeError, lambda: self.set ^ self.other)
        self.assertRaises(TypeError, lambda: self.other ^ self.set)
        if self.otherIsIterable:
            self.set.symmetric_difference(self.other)
        else:
            self.assertRaises(TypeError, self.set.symmetric_difference, self.other)

    def test_difference_update_operator(self):
        try:
            self.set -= self.other
        except TypeError:
            pass
        else:
            self.fail("expected TypeError")

    def test_difference_update(self):
        if self.otherIsIterable:
            self.set.difference_update(self.other)
        else:
            self.assertRaises(TypeError,
                              self.set.difference_update,
                              self.other)

    def test_difference(self):
        self.assertRaises(TypeError, lambda: self.set - self.other)
        self.assertRaises(TypeError, lambda: self.other - self.set)
        if self.otherIsIterable:
            self.set.difference(self.other)
        else:
            self.assertRaises(TypeError, self.set.difference, self.other)

#------------------------------------------------------------------------------

class TestOnlySetsNumeric(TestOnlySetsInBinaryOps):
    def setUp(self):
        self.set   = Set((1, 2, 3))
        self.other = 19
        self.otherIsIterable = False

#------------------------------------------------------------------------------

class TestOnlySetsDict(TestOnlySetsInBinaryOps):
    def setUp(self):
        self.set   = Set((1, 2, 3))
        self.other = {1:2, 3:4}
        self.otherIsIterable = True

#------------------------------------------------------------------------------

class TestOnlySetsOperator(TestOnlySetsInBinaryOps):
    def setUp(self):
        self.set   = Set((1, 2, 3))
        self.other = operator.add
        self.otherIsIterable = False

    def test_ge_gt_le_lt(self):
        with test_support.check_py3k_warnings():
            super(TestOnlySetsOperator, self).test_ge_gt_le_lt()

#------------------------------------------------------------------------------

class TestOnlySetsTuple(TestOnlySetsInBinaryOps):
    def setUp(self):
        self.set   = Set((1, 2, 3))
        self.other = (2, 4, 6)
        self.otherIsIterable = True

#------------------------------------------------------------------------------

class TestOnlySetsString(TestOnlySetsInBinaryOps):
    def setUp(self):
        self.set   = Set((1, 2, 3))
        self.other = 'abc'
        self.otherIsIterable = True

#------------------------------------------------------------------------------

class TestOnlySetsGenerator(TestOnlySetsInBinaryOps):
    def setUp(self):
        def gen():
            for i in xrange(0, 10, 2):
                yield i
        self.set   = Set((1, 2, 3))
        self.other = gen()
        self.otherIsIterable = True

#------------------------------------------------------------------------------

class TestOnlySetsofSets(TestOnlySetsInBinaryOps):
    def setUp(self):
        self.set   = Set((1, 2, 3))
        self.other = [Set('ab'), ImmutableSet('cd')]
        self.otherIsIterable = True

#==============================================================================

class TestCopying(unittest.TestCase):

    def test_copy(self):
        dup = self.set.copy()
        self.assertEqual(len(dup), len(self.set))
        dup_list = sorted(dup)
        set_list = sorted(self.set)
        self.assertEqual(len(dup_list), len(set_list))
        for i, el in enumerate(dup_list):
            self.assertIs(el, set_list[i])

    def test_deep_copy(self):
        dup = copy.deepcopy(self.set)
        self.assertSetEqual(dup, self.set)

#------------------------------------------------------------------------------

class TestCopyingEmpty(TestCopying):
    def setUp(self):
        self.set = Set()

#------------------------------------------------------------------------------

class TestCopyingSingleton(TestCopying):
    def setUp(self):
        self.set = Set(["hello"])

#------------------------------------------------------------------------------

class TestCopyingTriple(TestCopying):
    def setUp(self):
        self.set = Set(["zero", 0, None])

    def test_copy(self):
        with test_support.check_py3k_warnings():
            super(TestCopyingTriple, self).test_copy()

#------------------------------------------------------------------------------

class TestCopyingTuple(TestCopying):
    def setUp(self):
        self.set = Set([(1, 2)])

#------------------------------------------------------------------------------

class TestCopyingNested(TestCopying):
    def setUp(self):
        self.set = Set([((1, 2), (3, 4))])

#==============================================================================

class TestIdentities(unittest.TestCase):
    def setUp(self):
        self.a = Set([random.randrange(100) for i in xrange(50)])
        self.b = Set([random.randrange(100) for i in xrange(50)])

    def test_binopsVsSubsets(self):
        a, b = self.a, self.b
        self.assertTrue(a - b <= a)
        self.assertTrue(b - a <= b)
        self.assertTrue(a & b <= a)
        self.assertTrue(a & b <= b)
        self.assertTrue(a | b >= a)
        self.assertTrue(a | b >= b)
        self.assertTrue(a ^ b <= a | b)

    def test_commutativity(self):
        a, b = self.a, self.b
        self.assertEqual(a&b, b&a)
        self.assertEqual(a|b, b|a)
        self.assertEqual(a^b, b^a)
        if a != b:
            self.assertNotEqual(a-b, b-a)

    def test_reflexsive_relations(self):
        a, zero = self.a, Set()
        self.assertEqual(a ^ a, zero)
        self.assertEqual(a - a, zero)
        self.assertEqual(a | a, a)
        self.assertEqual(a & a, a)
        self.assertTrue(a <= a)
        self.assertTrue(a >= a)
        self.assertTrue(a == a)

    def test_summations(self):
        # check that sums of parts equal the whole
        a, b = self.a, self.b
        self.assertEqual((a-b)|(a&b)|(b-a), a|b)
        self.assertEqual((a&b)|(a^b), a|b)
        self.assertEqual(a|(b-a), a|b)
        self.assertEqual((a-b)|b, a|b)
        self.assertEqual((a-b)|(a&b), a)
        self.assertEqual((b-a)|(a&b), b)
        self.assertEqual((a-b)|(b-a), a^b)

    def test_exclusion(self):
        # check that inverse operations do not overlap
        a, b, zero = self.a, self.b, Set()
        self.assertEqual((a-b)&b, zero)
        self.assertEqual((b-a)&a, zero)
        self.assertEqual((a&b)&(a^b), zero)

    def test_cardinality_relations(self):
        a, b = self.a, self.b
        self.assertEqual(len(a), len(a-b) + len(a&b))
        self.assertEqual(len(b), len(b-a) + len(a&b))
        self.assertEqual(len(a^b), len(a-b) + len(b-a))
        self.assertEqual(len(a|b), len(a-b) + len(a&b) + len(b-a))
        self.assertEqual(len(a^b) + len(a&b), len(a|b))

#==============================================================================

libreftest = """
Example from the Library Reference:  Doc/lib/libsets.tex

>>> from sets import Set as Base  # override _repr to get sorted output
>>> class Set(Base):
...     def _repr(self):
...         return Base._repr(self, sorted=True)
>>> engineers = Set(['John', 'Jane', 'Jack', 'Janice'])
>>> programmers = Set(['Jack', 'Sam', 'Susan', 'Janice'])
>>> managers = Set(['Jane', 'Jack', 'Susan', 'Zack'])
>>> employees = engineers | programmers | managers           # union
>>> engineering_management = engineers & managers            # intersection
>>> fulltime_management = managers - engineers - programmers # difference
>>> engineers.add('Marvin')
>>> print engineers
Set(['Jack', 'Jane', 'Janice', 'John', 'Marvin'])
>>> employees.issuperset(engineers)           # superset test
False
>>> employees.union_update(engineers)         # update from another set
>>> employees.issuperset(engineers)
True
>>> for group in [engineers, programmers, managers, employees]:
...     group.discard('Susan')                # unconditionally remove element
...     print group
...
Set(['Jack', 'Jane', 'Janice', 'John', 'Marvin'])
Set(['Jack', 'Janice', 'Sam'])
Set(['Jack', 'Jane', 'Zack'])
Set(['Jack', 'Jane', 'Janice', 'John', 'Marvin', 'Sam', 'Zack'])
"""

#==============================================================================

__test__ = {'libreftest' : libreftest}

def test_main(verbose=None):
    import doctest
    from test import test_sets
    test_support.run_unittest(
        TestSetOfSets,
        TestExceptionPropagation,
        TestBasicOpsEmpty,
        TestBasicOpsSingleton,
        TestBasicOpsTuple,
        TestBasicOpsTriple,
        TestBinaryOps,
        TestUpdateOps,
        TestMutate,
        TestSubsetEqualEmpty,
        TestSubsetEqualNonEmpty,
        TestSubsetEmptyNonEmpty,
        TestSubsetPartial,
        TestSubsetNonOverlap,
        TestOnlySetsNumeric,
        TestOnlySetsDict,
        TestOnlySetsOperator,
        TestOnlySetsTuple,
        TestOnlySetsString,
        TestOnlySetsGenerator,
        TestOnlySetsofSets,
        TestCopyingEmpty,
        TestCopyingSingleton,
        TestCopyingTriple,
        TestCopyingTuple,
        TestCopyingNested,
        TestIdentities,
        doctest.DocTestSuite(test_sets),
    )

if __name__ == "__main__":
    test_main(verbose=True)



import math

from whatap.io.data_outputx import DataOutputX
from whatap.util.cardinality.murmurhash import MurMurHash
from whatap.util.cardinality.registerset import RegisterSet


def getAlphaMM(p, m):
    if p == 4:
        return 0.673 * m * m
    elif p == 5:
        return 0.697 * m * m
    elif p == 6:
        return 0.709 * m * m
    else:
        return (0.7213 / (1 + 1.079 / m)) * m * m


def linearCounting(m, V):
    return m * math.log(m / V)


def numberOfLeadingZeros(i):
    if not i:
        return 32

    n = 1
    if (i % 0x100000000) >> 16 == 0:
        n += 16
        i <<= 16
    if (i % 0x100000000) >> 24 == 0:
        n += 8
        i <<= 8
    if (i % 0x100000000) >> 28 == 0:
        n += 4
        i <<= 4
    if (i % 0x100000000) >> 30 == 0:
        n += 2
        i <<= 2

    n -= (i % 0x100000000) >> 31
    return n


class HyperLogLog(object):
    def __init__(self):
        self.log2m = 10
        self.registerSet = RegisterSet(1 << self.log2m)
        m = 1 << self.log2m
        self.alphaMM = getAlphaMM(self.log2m, m)

    def offerHashed(self, hashedValue):
        j = (hashedValue % 0x100000000) >> (32 - self.log2m)
        r = numberOfLeadingZeros(
            (hashedValue << self.log2m) | (1 << (self.log2m - 1)) + 1) + 1
        return self.registerSet.updateIfGreater(j, r)

    def offer(self, o):
        x = MurMurHash.hash(o)
        return self.offerHashed(x)

    def cardinality(self):
        registerSum = 0
        count = self.registerSet.count
        zeros = 0.0
        for j, _ in enumerate(count):
            val = self.registerSet.get(j)
            registerSum += 1.0 / (1 << val)
            if int(val) == 0:
                zeros += 1

        estimate = self.alphaMM * (1 / registerSum)
        if estimate <= (5.0 / 2.0) * count:
            return math.round(linearCounting(count, zeros))
        else:
            return math.round(estimate)

    def getBytes(self):
        dout = DataOutputX()
        dout.writeInt(self.log2m)
        dout.writeInt(self.registerSet.size)
        for m in self.registerSet.readOnlyBits():
            dout.writeInt(m)
        return dout.toByteArray()



# tk common message boxes
#
# this module provides an interface to the native message boxes
# available in Tk 4.2 and newer.
#
# written by Fredrik Lundh, May 1997
#

#
# options (all have default values):
#
# - default: which button to make default (one of the reply codes)
#
# - icon: which icon to display (see below)
#
# - message: the message to display
#
# - parent: which window to place the dialog on top of
#
# - title: dialog title
#
# - type: dialog type; that is, which buttons to display (see below)
#

from tkCommonDialog import Dialog

#
# constants

# icons
ERROR = "error"
INFO = "info"
QUESTION = "question"
WARNING = "warning"

# types
ABORTRETRYIGNORE = "abortretryignore"
OK = "ok"
OKCANCEL = "okcancel"
RETRYCANCEL = "retrycancel"
YESNO = "yesno"
YESNOCANCEL = "yesnocancel"

# replies
ABORT = "abort"
RETRY = "retry"
IGNORE = "ignore"
OK = "ok"
CANCEL = "cancel"
YES = "yes"
NO = "no"


#
# message dialog class

class Message(Dialog):
    "A message box"

    command  = "tk_messageBox"


#
# convenience stuff

# Rename _icon and _type options to allow overriding them in options
def _show(title=None, message=None, _icon=None, _type=None, **options):
    if _icon and "icon" not in options:    options["icon"] = _icon
    if _type and "type" not in options:    options["type"] = _type
    if title:   options["title"] = title
    if message: options["message"] = message
    res = Message(**options).show()
    # In some Tcl installations, yes/no is converted into a boolean.
    if isinstance(res, bool):
        if res:
            return YES
        return NO
    # In others we get a Tcl_Obj.
    return str(res)

def showinfo(title=None, message=None, **options):
    "Show an info message"
    return _show(title, message, INFO, OK, **options)

def showwarning(title=None, message=None, **options):
    "Show a warning message"
    return _show(title, message, WARNING, OK, **options)

def showerror(title=None, message=None, **options):
    "Show an error message"
    return _show(title, message, ERROR, OK, **options)

def askquestion(title=None, message=None, **options):
    "Ask a question"
    return _show(title, message, QUESTION, YESNO, **options)

def askokcancel(title=None, message=None, **options):
    "Ask if operation should proceed; return true if the answer is ok"
    s = _show(title, message, QUESTION, OKCANCEL, **options)
    return s == OK

def askyesno(title=None, message=None, **options):
    "Ask a question; return true if the answer is yes"
    s = _show(title, message, QUESTION, YESNO, **options)
    return s == YES

def askyesnocancel(title=None, message=None, **options):
    "Ask a question; return true if the answer is yes, None if cancelled."
    s = _show(title, message, QUESTION, YESNOCANCEL, **options)
    # s might be a Tcl index object, so convert it to a string
    s = str(s)
    if s == CANCEL:
        return None
    return s == YES

def askretrycancel(title=None, message=None, **options):
    "Ask if operation should be retried; return true if the answer is yes"
    s = _show(title, message, WARNING, RETRYCANCEL, **options)
    return s == RETRY


# --------------------------------------------------------------------
# test stuff

if __name__ == "__main__":

    print "info", showinfo("Spam", "Egg Information")
    print "warning", showwarning("Spam", "Egg Warning")
    print "error", showerror("Spam", "Egg Alert")
    print "question", askquestion("Spam", "Question?")
    print "proceed", askokcancel("Spam", "Proceed?")
    print "yes/no", askyesno("Spam", "Got it?")
    print "yes/no/cancel", askyesnocancel("Spam", "Want it?")
    print "try again", askretrycancel("Spam", "Try again?")



import whatap.agent.conf.configure as conf
import whatap.util.logging_util as logging_util
import sys

class TaskConfig:
    def __init__(self):
	    self.process(None, None, None, None)

    def interval(self):
        return 15

    def process(self, os_ , pcode , oid , now ) :
        conf.reload()
        logging_util.isDebug = conf.GetConfig().Debug

        if not conf.GetConfig().Enabled:
            logging_util.info("Received Agent Status Closed received. Shutting down...")
            sys.exit(0)

        return None



import math
import unittest
import sys
import _ast
from test import test_support
from test import script_helper
import os
import tempfile
import textwrap

class TestSpecifics(unittest.TestCase):

    def test_no_ending_newline(self):
        compile("hi", "<test>", "exec")
        compile("hi\r", "<test>", "exec")

    def test_empty(self):
        compile("", "<test>", "exec")

    def test_other_newlines(self):
        compile("\r\n", "<test>", "exec")
        compile("\r", "<test>", "exec")
        compile("hi\r\nstuff\r\ndef f():\n    pass\r", "<test>", "exec")
        compile("this_is\rreally_old_mac\rdef f():\n    pass", "<test>", "exec")

    def test_debug_assignment(self):
        # catch assignments to __debug__
        self.assertRaises(SyntaxError, compile, '__debug__ = 1', '?', 'single')
        import __builtin__
        prev = __builtin__.__debug__
        setattr(__builtin__, '__debug__', 'sure')
        setattr(__builtin__, '__debug__', prev)

    def test_argument_handling(self):
        # detect duplicate positional and keyword arguments
        self.assertRaises(SyntaxError, eval, 'lambda a,a:0')
        self.assertRaises(SyntaxError, eval, 'lambda a,a=1:0')
        self.assertRaises(SyntaxError, eval, 'lambda a=1,a=1:0')
        try:
            exec 'def f(a, a): pass'
            self.fail("duplicate arguments")
        except SyntaxError:
            pass
        try:
            exec 'def f(a = 0, a = 1): pass'
            self.fail("duplicate keyword arguments")
        except SyntaxError:
            pass
        try:
            exec 'def f(a): global a; a = 1'
            self.fail("variable is global and local")
        except SyntaxError:
            pass

    def test_syntax_error(self):
        self.assertRaises(SyntaxError, compile, "1+*3", "filename", "exec")

    def test_none_keyword_arg(self):
        self.assertRaises(SyntaxError, compile, "f(None=1)", "<string>", "exec")

    def test_duplicate_global_local(self):
        try:
            exec 'def f(a): global a; a = 1'
            self.fail("variable is global and local")
        except SyntaxError:
            pass

    def test_exec_functional_style(self):
        # Exec'ing a tuple of length 2 works.
        g = {'b': 2}
        exec("a = b + 1", g)
        self.assertEqual(g['a'], 3)

        # As does exec'ing a tuple of length 3.
        l = {'b': 3}
        g = {'b': 5, 'c': 7}
        exec("a = b + c", g, l)
        self.assertNotIn('a', g)
        self.assertEqual(l['a'], 10)

        # Tuples not of length 2 or 3 are invalid.
        with self.assertRaises(TypeError):
            exec("a = b + 1",)

        with self.assertRaises(TypeError):
            exec("a = b + 1", {}, {}, {})

        # Can't mix and match the two calling forms.
        g = {'a': 3, 'b': 4}
        l = {}
        with self.assertRaises(TypeError):
            exec("a = b + 1", g) in g
        with self.assertRaises(TypeError):
            exec("a = b + 1", g, l) in g, l

    def test_nested_qualified_exec(self):
        # Can use qualified exec in nested functions.
        code = ["""
def g():
    def f():
        if True:
            exec "" in {}, {}
        """, """
def g():
    def f():
        if True:
            exec("", {}, {})
        """, """
def g():
    def f():
        if True:
            exec("", {})
        """]
        for c in code:
            compile(c, "<code>", "exec")

    def test_exec_with_general_mapping_for_locals(self):

        class M:
            "Test mapping interface versus possible calls from eval()."
            def __getitem__(self, key):
                if key == 'a':
                    return 12
                raise KeyError
            def __setitem__(self, key, value):
                self.results = (key, value)
            def keys(self):
                return list('xyz')

        m = M()
        g = globals()
        exec 'z = a' in g, m
        self.assertEqual(m.results, ('z', 12))
        try:
            exec 'z = b' in g, m
        except NameError:
            pass
        else:
            self.fail('Did not detect a KeyError')
        exec 'z = dir()' in g, m
        self.assertEqual(m.results, ('z', list('xyz')))
        exec 'z = globals()' in g, m
        self.assertEqual(m.results, ('z', g))
        exec 'z = locals()' in g, m
        self.assertEqual(m.results, ('z', m))
        try:
            exec 'z = b' in m
        except TypeError:
            pass
        else:
            self.fail('Did not validate globals as a real dict')

        class A:
            "Non-mapping"
            pass
        m = A()
        try:
            exec 'z = a' in g, m
        except TypeError:
            pass
        else:
            self.fail('Did not validate locals as a mapping')

        # Verify that dict subclasses work as well
        class D(dict):
            def __getitem__(self, key):
                if key == 'a':
                    return 12
                return dict.__getitem__(self, key)
        d = D()
        exec 'z = a' in g, d
        self.assertEqual(d['z'], 12)

    def test_extended_arg(self):
        longexpr = 'x = x or ' + '-x' * 2500
        code = '''
def f(x):
    %s
    %s
    %s
    %s
    %s
    %s
    %s
    %s
    %s
    %s
    # the expressions above have no effect, x == argument
    while x:
        x -= 1
        # EXTENDED_ARG/JUMP_ABSOLUTE here
    return x
''' % ((longexpr,)*10)
        exec code
        self.assertEqual(f(5), 0)

    def test_complex_args(self):

        with test_support.check_py3k_warnings(
                ("tuple parameter unpacking has been removed", SyntaxWarning)):
            exec textwrap.dedent('''
        def comp_args((a, b)):
            return a,b
        self.assertEqual(comp_args((1, 2)), (1, 2))

        def comp_args((a, b)=(3, 4)):
            return a, b
        self.assertEqual(comp_args((1, 2)), (1, 2))
        self.assertEqual(comp_args(), (3, 4))

        def comp_args(a, (b, c)):
            return a, b, c
        self.assertEqual(comp_args(1, (2, 3)), (1, 2, 3))

        def comp_args(a=2, (b, c)=(3, 4)):
            return a, b, c
        self.assertEqual(comp_args(1, (2, 3)), (1, 2, 3))
        self.assertEqual(comp_args(), (2, 3, 4))
        ''')

    def test_argument_order(self):
        try:
            exec 'def f(a=1, (b, c)): pass'
            self.fail("non-default args after default")
        except SyntaxError:
            pass

    def test_float_literals(self):
        # testing bad float literals
        self.assertRaises(SyntaxError, eval, "2e")
        self.assertRaises(SyntaxError, eval, "2.0e+")
        self.assertRaises(SyntaxError, eval, "1e-")
        self.assertRaises(SyntaxError, eval, "3-4e/21")

    def test_indentation(self):
        # testing compile() of indented block w/o trailing newline"
        s = """
if 1:
    if 2:
        pass"""
        compile(s, "<string>", "exec")

    # This test is probably specific to CPython and may not generalize
    # to other implementations.  We are trying to ensure that when
    # the first line of code starts after 256, correct line numbers
    # in tracebacks are still produced.
    def test_leading_newlines(self):
        s256 = "".join(["\n"] * 256 + ["spam"])
        co = compile(s256, 'fn', 'exec')
        self.assertEqual(co.co_firstlineno, 257)
        self.assertEqual(co.co_lnotab, '')

    def test_literals_with_leading_zeroes(self):
        for arg in ["077787", "0xj", "0x.", "0e",  "090000000000000",
                    "080000000000000", "000000000000009", "000000000000008",
                    "0b42", "0BADCAFE", "0o123456789", "0b1.1", "0o4.2",
                    "0b101j2", "0o153j2", "0b100e1", "0o777e1", "0o8", "0o78"]:
            self.assertRaises(SyntaxError, eval, arg)

        self.assertEqual(eval("0777"), 511)
        self.assertEqual(eval("0777L"), 511)
        self.assertEqual(eval("000777"), 511)
        self.assertEqual(eval("0xff"), 255)
        self.assertEqual(eval("0xffL"), 255)
        self.assertEqual(eval("0XfF"), 255)
        self.assertEqual(eval("0777."), 777)
        self.assertEqual(eval("0777.0"), 777)
        self.assertEqual(eval("000000000000000000000000000000000000000000000000000777e0"), 777)
        self.assertEqual(eval("0777e1"), 7770)
        self.assertEqual(eval("0e0"), 0)
        self.assertEqual(eval("0000E-012"), 0)
        self.assertEqual(eval("09.5"), 9.5)
        self.assertEqual(eval("0777j"), 777j)
        self.assertEqual(eval("00j"), 0j)
        self.assertEqual(eval("00.0"), 0)
        self.assertEqual(eval("0e3"), 0)
        self.assertEqual(eval("090000000000000."), 90000000000000.)
        self.assertEqual(eval("090000000000000.0000000000000000000000"), 90000000000000.)
        self.assertEqual(eval("090000000000000e0"), 90000000000000.)
        self.assertEqual(eval("090000000000000e-0"), 90000000000000.)
        self.assertEqual(eval("090000000000000j"), 90000000000000j)
        self.assertEqual(eval("000000000000007"), 7)
        self.assertEqual(eval("000000000000008."), 8.)
        self.assertEqual(eval("000000000000009."), 9.)
        self.assertEqual(eval("0b101010"), 42)
        self.assertEqual(eval("-0b000000000010"), -2)
        self.assertEqual(eval("0o777"), 511)
        self.assertEqual(eval("-0o0000010"), -8)
        self.assertEqual(eval("020000000000.0"), 20000000000.0)
        self.assertEqual(eval("037777777777e0"), 37777777777.0)
        self.assertEqual(eval("01000000000000000000000.0"),
                         1000000000000000000000.0)

    def test_unary_minus(self):
        # Verify treatment of unary minus on negative numbers SF bug #660455
        if sys.maxint == 2147483647:
            # 32-bit machine
            all_one_bits = '0xffffffff'
            self.assertEqual(eval(all_one_bits), 4294967295L)
            self.assertEqual(eval("-" + all_one_bits), -4294967295L)
        elif sys.maxint == 9223372036854775807:
            # 64-bit machine
            all_one_bits = '0xffffffffffffffff'
            self.assertEqual(eval(all_one_bits), 18446744073709551615L)
            self.assertEqual(eval("-" + all_one_bits), -18446744073709551615L)
        else:
            self.fail("How many bits *does* this machine have???")
        # Verify treatment of constant folding on -(sys.maxint+1)
        # i.e. -2147483648 on 32 bit platforms.  Should return int, not long.
        self.assertIsInstance(eval("%s" % (-sys.maxint - 1)), int)
        self.assertIsInstance(eval("%s" % (-sys.maxint - 2)), long)

    if sys.maxint == 9223372036854775807:
        def test_32_63_bit_values(self):
            a = +4294967296  # 1 << 32
            b = -4294967296  # 1 << 32
            c = +281474976710656  # 1 << 48
            d = -281474976710656  # 1 << 48
            e = +4611686018427387904  # 1 << 62
            f = -4611686018427387904  # 1 << 62
            g = +9223372036854775807  # 1 << 63 - 1
            h = -9223372036854775807  # 1 << 63 - 1

            for variable in self.test_32_63_bit_values.func_code.co_consts:
                if variable is not None:
                    self.assertIsInstance(variable, int)

    def test_sequence_unpacking_error(self):
        # Verify sequence packing/unpacking with "or".  SF bug #757818
        i,j = (1, -1) or (-1, 1)
        self.assertEqual(i, 1)
        self.assertEqual(j, -1)

    def test_none_assignment(self):
        stmts = [
            'None = 0',
            'None += 0',
            '__builtins__.None = 0',
            'def None(): pass',
            'class None: pass',
            '(a, None) = 0, 0',
            'for None in range(10): pass',
            'def f(None): pass',
            'import None',
            'import x as None',
            'from x import None',
            'from x import y as None'
        ]
        for stmt in stmts:
            stmt += "\n"
            self.assertRaises(SyntaxError, compile, stmt, 'tmp', 'single')
            self.assertRaises(SyntaxError, compile, stmt, 'tmp', 'exec')
        # This is ok.
        compile("from None import x", "tmp", "exec")
        compile("from x import None as y", "tmp", "exec")
        compile("import None as x", "tmp", "exec")

    def test_import(self):
        succeed = [
            'import sys',
            'import os, sys',
            'import os as bar',
            'import os.path as bar',
            'from __future__ import nested_scopes, generators',
            'from __future__ import (nested_scopes,\ngenerators)',
            'from __future__ import (nested_scopes,\ngenerators,)',
            'from sys import stdin, stderr, stdout',
            'from sys import (stdin, stderr,\nstdout)',
            'from sys import (stdin, stderr,\nstdout,)',
            'from sys import (stdin\n, stderr, stdout)',
            'from sys import (stdin\n, stderr, stdout,)',
            'from sys import stdin as si, stdout as so, stderr as se',
            'from sys import (stdin as si, stdout as so, stderr as se)',
            'from sys import (stdin as si, stdout as so, stderr as se,)',
            ]
        fail = [
            'import (os, sys)',
            'import (os), (sys)',
            'import ((os), (sys))',
            'import (sys',
            'import sys)',
            'import (os,)',
            'import os As bar',
            'import os.path a bar',
            'from sys import stdin As stdout',
            'from sys import stdin a stdout',
            'from (sys) import stdin',
            'from __future__ import (nested_scopes',
            'from __future__ import nested_scopes)',
            'from __future__ import nested_scopes,\ngenerators',
            'from sys import (stdin',
            'from sys import stdin)',
            'from sys import stdin, stdout,\nstderr',
            'from sys import stdin si',
            'from sys import stdin,',
            'from sys import (*)',
            'from sys import (stdin,, stdout, stderr)',
            'from sys import (stdin, stdout),',
            ]
        for stmt in succeed:
            compile(stmt, 'tmp', 'exec')
        for stmt in fail:
            self.assertRaises(SyntaxError, compile, stmt, 'tmp', 'exec')

    def test_for_distinct_code_objects(self):
        # SF bug 1048870
        def f():
            f1 = lambda x=1: x
            f2 = lambda x=2: x
            return f1, f2
        f1, f2 = f()
        self.assertNotEqual(id(f1.func_code), id(f2.func_code))

    def test_lambda_doc(self):
        l = lambda: "foo"
        self.assertIsNone(l.__doc__)

    @test_support.requires_unicode
    def test_encoding(self):
        code = b'# -*- coding: badencoding -*-\npass\n'
        self.assertRaises(SyntaxError, compile, code, 'tmp', 'exec')
        code = u"# -*- coding: utf-8 -*-\npass\n"
        self.assertRaises(SyntaxError, compile, code, "tmp", "exec")
        code = 'u"\xc2\xa4"\n'
        self.assertEqual(eval(code), u'\xc2\xa4')
        code = u'u"\xc2\xa4"\n'
        self.assertEqual(eval(code), u'\xc2\xa4')
        code = '# -*- coding: latin1 -*-\nu"\xc2\xa4"\n'
        self.assertEqual(eval(code), u'\xc2\xa4')
        code = '# -*- coding: utf-8 -*-\nu"\xc2\xa4"\n'
        self.assertEqual(eval(code), u'\xa4')
        code = '# -*- coding: iso8859-15 -*-\nu"\xc2\xa4"\n'
        self.assertEqual(eval(code), test_support.u(r'\xc2\u20ac'))
        code = 'u"""\\\n# -*- coding: utf-8 -*-\n\xc2\xa4"""\n'
        self.assertEqual(eval(code), u'# -*- coding: utf-8 -*-\n\xc2\xa4')

    def test_subscripts(self):
        # SF bug 1448804
        # Class to make testing subscript results easy
        class str_map(object):
            def __init__(self):
                self.data = {}
            def __getitem__(self, key):
                return self.data[str(key)]
            def __setitem__(self, key, value):
                self.data[str(key)] = value
            def __delitem__(self, key):
                del self.data[str(key)]
            def __contains__(self, key):
                return str(key) in self.data
        d = str_map()
        # Index
        d[1] = 1
        self.assertEqual(d[1], 1)
        d[1] += 1
        self.assertEqual(d[1], 2)
        del d[1]
        self.assertNotIn(1, d)
        # Tuple of indices
        d[1, 1] = 1
        self.assertEqual(d[1, 1], 1)
        d[1, 1] += 1
        self.assertEqual(d[1, 1], 2)
        del d[1, 1]
        self.assertNotIn((1, 1), d)
        # Simple slice
        d[1:2] = 1
        self.assertEqual(d[1:2], 1)
        d[1:2] += 1
        self.assertEqual(d[1:2], 2)
        del d[1:2]
        self.assertNotIn(slice(1, 2), d)
        # Tuple of simple slices
        d[1:2, 1:2] = 1
        self.assertEqual(d[1:2, 1:2], 1)
        d[1:2, 1:2] += 1
        self.assertEqual(d[1:2, 1:2], 2)
        del d[1:2, 1:2]
        self.assertNotIn((slice(1, 2), slice(1, 2)), d)
        # Extended slice
        d[1:2:3] = 1
        self.assertEqual(d[1:2:3], 1)
        d[1:2:3] += 1
        self.assertEqual(d[1:2:3], 2)
        del d[1:2:3]
        self.assertNotIn(slice(1, 2, 3), d)
        # Tuple of extended slices
        d[1:2:3, 1:2:3] = 1
        self.assertEqual(d[1:2:3, 1:2:3], 1)
        d[1:2:3, 1:2:3] += 1
        self.assertEqual(d[1:2:3, 1:2:3], 2)
        del d[1:2:3, 1:2:3]
        self.assertNotIn((slice(1, 2, 3), slice(1, 2, 3)), d)
        # Ellipsis
        d[...] = 1
        self.assertEqual(d[...], 1)
        d[...] += 1
        self.assertEqual(d[...], 2)
        del d[...]
        self.assertNotIn(Ellipsis, d)
        # Tuple of Ellipses
        d[..., ...] = 1
        self.assertEqual(d[..., ...], 1)
        d[..., ...] += 1
        self.assertEqual(d[..., ...], 2)
        del d[..., ...]
        self.assertNotIn((Ellipsis, Ellipsis), d)

    def test_mangling(self):
        class A:
            def f():
                __mangled = 1
                __not_mangled__ = 2
                import __mangled_mod
                import __package__.module

        self.assertIn("_A__mangled", A.f.func_code.co_varnames)
        self.assertIn("__not_mangled__", A.f.func_code.co_varnames)
        self.assertIn("_A__mangled_mod", A.f.func_code.co_varnames)
        self.assertIn("__package__", A.f.func_code.co_varnames)

    def test_compile_ast(self):
        fname = __file__
        if fname.lower().endswith(('pyc', 'pyo')):
            fname = fname[:-1]
        with open(fname, 'r') as f:
            fcontents = f.read()
        sample_code = [
            ['<assign>', 'x = 5'],
            ['<print1>', 'print 1'],
            ['<printv>', 'print v'],
            ['<printTrue>', 'print True'],
            ['<printList>', 'print []'],
            ['<ifblock>', """if True:\n    pass\n"""],
            ['<forblock>', """for n in [1, 2, 3]:\n    print n\n"""],
            ['<deffunc>', """def foo():\n    pass\nfoo()\n"""],
            [fname, fcontents],
        ]

        for fname, code in sample_code:
            co1 = compile(code, '%s1' % fname, 'exec')
            ast = compile(code, '%s2' % fname, 'exec', _ast.PyCF_ONLY_AST)
            self.assertTrue(type(ast) == _ast.Module)
            co2 = compile(ast, '%s3' % fname, 'exec')
            self.assertEqual(co1, co2)
            # the code object's filename comes from the second compilation step
            self.assertEqual(co2.co_filename, '%s3' % fname)

        # raise exception when node type doesn't match with compile mode
        co1 = compile('print 1', '<string>', 'exec', _ast.PyCF_ONLY_AST)
        self.assertRaises(TypeError, compile, co1, '<ast>', 'eval')

        # raise exception when node type is no start node
        self.assertRaises(TypeError, compile, _ast.If(), '<ast>', 'exec')

        # raise exception when node has invalid children
        ast = _ast.Module()
        ast.body = [_ast.BoolOp()]
        self.assertRaises(TypeError, compile, ast, '<ast>', 'exec')

    def test_yet_more_evil_still_undecodable(self):
        # Issue #25388
        src = b"#\x00\n#\xfd\n"
        tmpd = tempfile.mkdtemp()
        try:
            fn = os.path.join(tmpd, "bad.py")
            with open(fn, "wb") as fp:
                fp.write(src)
            rc, out, err = script_helper.assert_python_failure(fn)
        finally:
            test_support.rmtree(tmpd)
        self.assertIn(b"Non-ASCII", err)

    def test_null_terminated(self):
        # The source code is null-terminated internally, but bytes-like
        # objects are accepted, which could be not terminated.
        with self.assertRaisesRegexp(TypeError, "without null bytes"):
            compile(u"123\x00", "<dummy>", "eval")
        with test_support.check_py3k_warnings():
            with self.assertRaisesRegexp(TypeError, "without null bytes"):
                compile(buffer("123\x00"), "<dummy>", "eval")
            code = compile(buffer("123\x00", 1, 2), "<dummy>", "eval")
            self.assertEqual(eval(code), 23)
            code = compile(buffer("1234", 1, 2), "<dummy>", "eval")
            self.assertEqual(eval(code), 23)
            code = compile(buffer("$23$", 1, 2), "<dummy>", "eval")
            self.assertEqual(eval(code), 23)

class TestStackSize(unittest.TestCase):
    # These tests check that the computed stack size for a code object
    # stays within reasonable bounds (see issue #21523 for an example
    # dysfunction).
    N = 100

    def check_stack_size(self, code):
        # To assert that the alleged stack size is not O(N), we
        # check that it is smaller than log(N).
        if isinstance(code, str):
            code = compile(code, "<foo>", "single")
        max_size = math.ceil(math.log(len(code.co_code)))
        self.assertLessEqual(code.co_stacksize, max_size)

    def test_and(self):
        self.check_stack_size("x and " * self.N + "x")

    def test_or(self):
        self.check_stack_size("x or " * self.N + "x")

    def test_and_or(self):
        self.check_stack_size("x and x or " * self.N + "x")

    def test_chained_comparison(self):
        self.check_stack_size("x < " * self.N + "x")

    def test_if_else(self):
        self.check_stack_size("x if x else " * self.N + "x")

    def test_binop(self):
        self.check_stack_size("x + " * self.N + "x")

    def test_func_and(self):
        code = "def f(x):\n"
        code += "   x and x\n" * self.N
        self.check_stack_size(code)

    def check_constant(self, func, expected):
        for const in func.__code__.co_consts:
            if repr(const) == repr(expected):
                break
        else:
            self.fail("unable to find constant %r in %r"
                      % (expected, func.__code__.co_consts))

    # Merging equal constants is not a strict requirement for the Python
    # semantics, it's a more an implementation detail.
    @test_support.cpython_only
    def test_merge_constants(self):
        # Issue #25843: compile() must merge constants which are equal
        # and have the same type.

        def check_same_constant(const):
            ns = {}
            code = "f1, f2 = lambda: %r, lambda: %r" % (const, const)
            exec(code, ns)
            f1 = ns['f1']
            f2 = ns['f2']
            self.assertIs(f1.__code__, f2.__code__)
            self.check_constant(f1, const)
            self.assertEqual(repr(f1()), repr(const))

        check_same_constant(None)
        check_same_constant(0)
        check_same_constant(0.0)
        check_same_constant(b'abc')
        check_same_constant('abc')

    def test_dont_merge_constants(self):
        # Issue #25843: compile() must not merge constants which are equal
        # but have a different type.

        def check_different_constants(const1, const2):
            ns = {}
            exec("f1, f2 = lambda: %r, lambda: %r" % (const1, const2), ns)
            f1 = ns['f1']
            f2 = ns['f2']
            self.assertIsNot(f1.__code__, f2.__code__)
            self.check_constant(f1, const1)
            self.check_constant(f2, const2)
            self.assertEqual(repr(f1()), repr(const1))
            self.assertEqual(repr(f2()), repr(const2))

        check_different_constants(0, 0.0)
        check_different_constants(+0.0, -0.0)
        check_different_constants((0,), (0.0,))

        # check_different_constants() cannot be used because repr(-0j) is
        # '(-0-0j)', but when '(-0-0j)' is evaluated to 0j: we loose the sign.
        f1, f2 = lambda: +0.0j, lambda: -0.0j
        self.assertIsNot(f1.__code__, f2.__code__)
        self.check_constant(f1, +0.0j)
        self.check_constant(f2, -0.0j)
        self.assertEqual(repr(f1()), repr(+0.0j))
        self.assertEqual(repr(f2()), repr(-0.0j))


def test_main():
    test_support.run_unittest(__name__)

if __name__ == "__main__":
    unittest.main()



import ctypes
import ctypes.util
import platform


_libcrypto = None
_signatures_set = False
_LOAD_ERROR = None

_USE_LEGACY_CTX = False

_LEGACY_CTX_BUF_SIZE = 4096


def _candidate_paths():
    sysname = platform.system().lower()
    cands = []

    try:
        name = ctypes.util.find_library('crypto')
        if name:
            cands.append(name)
    except Exception:
        pass

    if 'aix' in sysname:
        import struct as _struct
        is_64bit = _struct.calcsize("P") == 8
        archives = [
            '/usr/lib/libcrypto.a',
            '/opt/freeware/lib/libcrypto.a',
            '/usr/local/lib/libcrypto.a',
        ]
        members = []
        if is_64bit:
            members.extend([
                'libcrypto64.so.3',
                'libcrypto64.so.1.1',
                'libcrypto64.so.1.0.2',
                'libcrypto64.so.1.0.0',
                'libcrypto64.so',
            ])
        members.extend([
            'libcrypto.so.3',
            'libcrypto.so.1.1',
            'libcrypto.so.1.0.2',
            'libcrypto.so.1.0.0',
            'libcrypto.so',
        ])
        for ar in archives:
            for mem in members:
                cands.append('%s(%s)' % (ar, mem))
    elif 'hp-ux' in sysname or 'hpux' in sysname:
        import struct as _struct
        is_64bit = _struct.calcsize("P") == 8
        if is_64bit:
            cands.extend([
            ])
        else:
            cands.extend([
            ])
        cands.extend([
        ])
    elif 'sunos' in sysname or 'solaris' in sysname:
        cands.extend([
            '/usr/lib/64/libcrypto.so',
            '/usr/lib/libcrypto.so',
            '/usr/sfw/lib/64/libcrypto.so',
            '/usr/sfw/lib/libcrypto.so',
            '/opt/csw/lib/libcrypto.so',
        ])
    elif 'darwin' in sysname:
        cands.extend([
            '/usr/lib/libcrypto.dylib',
            '/opt/homebrew/lib/libcrypto.dylib',
            '/usr/local/opt/openssl/lib/libcrypto.dylib',
            '/usr/local/lib/libcrypto.dylib',
        ])
    else:
        cands.extend([
            'libcrypto.so.3',
            'libcrypto.so.1.1',
            'libcrypto.so.1.0.0',
            'libcrypto.so.10',
            'libcrypto.so',
        ])

    return cands, sysname


_AIX_RTLD_NOW = 0x00000002
_AIX_RTLD_MEMBER = 0x00040000


def _try_load():
    global _libcrypto, _LOAD_ERROR
    if _libcrypto is not None:
        return _libcrypto

    candidates, sysname = _candidate_paths()
    is_aix = 'aix' in sysname
    rtld_member_val = getattr(ctypes, 'RTLD_MEMBER', None)
    if rtld_member_val is None:
        rtld_member_val = _AIX_RTLD_MEMBER
    rtld_now_val = getattr(ctypes, 'RTLD_NOW', None)
    if rtld_now_val is None:
        rtld_now_val = _AIX_RTLD_NOW

    attempts = []  # list of (path, error) for diagnostics

    for cand in candidates:
        try:
            if is_aix and '(' in cand:
                lib = ctypes.CDLL(cand, mode=rtld_now_val | rtld_member_val)
            else:
                lib = ctypes.CDLL(cand)
            _libcrypto = lib
            return lib
        except OSError as e:
            attempts.append((cand, str(e)))
            continue
        except Exception as e:
            attempts.append((cand, "%s: %s" % (type(e).__name__, e)))
            continue

    if attempts:
        tail = attempts[-5:]
        _LOAD_ERROR = "tried %d paths, last failures: %s" % (
            len(attempts),
            "; ".join("%s -> %s" % (p, e) for p, e in tail),
        )
    else:
        _LOAD_ERROR = "no candidate paths to try"
    return None


def _setup_signatures(lib):
    global _USE_LEGACY_CTX

    has_new = hasattr(lib, 'EVP_CIPHER_CTX_new')
    has_init = hasattr(lib, 'EVP_CIPHER_CTX_init')

    if has_new:
        lib.EVP_CIPHER_CTX_new.restype = ctypes.c_void_p
        lib.EVP_CIPHER_CTX_new.argtypes = []
        lib.EVP_CIPHER_CTX_free.argtypes = [ctypes.c_void_p]
        lib.EVP_CIPHER_CTX_free.restype = None
        _USE_LEGACY_CTX = False
    elif has_init:
        lib.EVP_CIPHER_CTX_init.argtypes = [ctypes.c_void_p]
        lib.EVP_CIPHER_CTX_init.restype = None
        if hasattr(lib, 'EVP_CIPHER_CTX_cleanup'):
            lib.EVP_CIPHER_CTX_cleanup.argtypes = [ctypes.c_void_p]
            lib.EVP_CIPHER_CTX_cleanup.restype = ctypes.c_int
        _USE_LEGACY_CTX = True
    else:
        raise RuntimeError(
            "neither EVP_CIPHER_CTX_new nor EVP_CIPHER_CTX_init available")

    lib.EVP_aes_128_ecb.restype = ctypes.c_void_p
    lib.EVP_aes_128_ecb.argtypes = []
    lib.EVP_aes_192_ecb.restype = ctypes.c_void_p
    lib.EVP_aes_192_ecb.argtypes = []
    lib.EVP_aes_256_ecb.restype = ctypes.c_void_p
    lib.EVP_aes_256_ecb.argtypes = []

    lib.EVP_EncryptInit_ex.argtypes = [
        ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p,
        ctypes.c_char_p, ctypes.c_char_p,
    ]
    lib.EVP_EncryptInit_ex.restype = ctypes.c_int
    lib.EVP_EncryptUpdate.argtypes = [
        ctypes.c_void_p, ctypes.c_void_p, ctypes.POINTER(ctypes.c_int),
        ctypes.c_char_p, ctypes.c_int,
    ]
    lib.EVP_EncryptUpdate.restype = ctypes.c_int
    lib.EVP_EncryptFinal_ex.argtypes = [
        ctypes.c_void_p, ctypes.c_void_p, ctypes.POINTER(ctypes.c_int),
    ]
    lib.EVP_EncryptFinal_ex.restype = ctypes.c_int

    lib.EVP_DecryptInit_ex.argtypes = [
        ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p,
        ctypes.c_char_p, ctypes.c_char_p,
    ]
    lib.EVP_DecryptInit_ex.restype = ctypes.c_int
    lib.EVP_DecryptUpdate.argtypes = [
        ctypes.c_void_p, ctypes.c_void_p, ctypes.POINTER(ctypes.c_int),
        ctypes.c_char_p, ctypes.c_int,
    ]
    lib.EVP_DecryptUpdate.restype = ctypes.c_int
    lib.EVP_DecryptFinal_ex.argtypes = [
        ctypes.c_void_p, ctypes.c_void_p, ctypes.POINTER(ctypes.c_int),
    ]
    lib.EVP_DecryptFinal_ex.restype = ctypes.c_int

    lib.EVP_CIPHER_CTX_set_padding.argtypes = [ctypes.c_void_p, ctypes.c_int]
    lib.EVP_CIPHER_CTX_set_padding.restype = ctypes.c_int


def is_available():
    global _signatures_set
    lib = _try_load()
    if lib is None:
        return False
    if not _signatures_set:
        try:
            _setup_signatures(lib)
            _signatures_set = True
        except Exception as e:
            global _LOAD_ERROR
            _LOAD_ERROR = e
            return False
    return True


def get_load_error():
    return _LOAD_ERROR


def get_library_path():
    if _libcrypto is None:
        return None
    name = getattr(_libcrypto, '_name', None)
    return name or repr(_libcrypto)


class LibcryptoAES_ECB(object):
    def __init__(self, key):
        if not is_available():
            raise RuntimeError("libcrypto unavailable: %s" % _LOAD_ERROR)

        if isinstance(key, bytearray):
            key = bytes(key)
        elif not isinstance(key, bytes):
            key = bytes(key)

        keylen = len(key)
        if keylen == 16:
            self._cipher = _libcrypto.EVP_aes_128_ecb()
        elif keylen == 24:
            self._cipher = _libcrypto.EVP_aes_192_ecb()
        elif keylen == 32:
            self._cipher = _libcrypto.EVP_aes_256_ecb()
        else:
            raise ValueError(
                "AES key must be 16, 24, or 32 bytes (got %d)" % keylen)

        if not self._cipher:
            raise RuntimeError("EVP_aes_*_ecb returned NULL")

        self._key = key

    def encrypt(self, data):
        return self._run(data, encrypt_mode=True)

    def decrypt(self, data):
        return self._run(data, encrypt_mode=False)

    def _run(self, data, encrypt_mode):
        if data is None:
            return data
        if isinstance(data, bytearray):
            data = bytes(data)
        elif not isinstance(data, bytes):
            data = bytes(data)

        in_len = len(data)
        if in_len == 0:
            return b''
        if in_len % 16 != 0:
            raise ValueError(
                "data length must be a multiple of 16 (got %d)" % in_len)

        ctx_buf = None
        if _USE_LEGACY_CTX:
            ctx_buf = ctypes.create_string_buffer(_LEGACY_CTX_BUF_SIZE)
            ctx = ctx_buf
            _libcrypto.EVP_CIPHER_CTX_init(ctx)
        else:
            ctx = _libcrypto.EVP_CIPHER_CTX_new()
            if not ctx:
                raise RuntimeError("EVP_CIPHER_CTX_new failed")
        try:
            if encrypt_mode:
                rc = _libcrypto.EVP_EncryptInit_ex(
                    ctx, self._cipher, None, self._key, None)
                if rc != 1:
                    raise RuntimeError("EVP_EncryptInit_ex failed")
            else:
                rc = _libcrypto.EVP_DecryptInit_ex(
                    ctx, self._cipher, None, self._key, None)
                if rc != 1:
                    raise RuntimeError("EVP_DecryptInit_ex failed")

            _libcrypto.EVP_CIPHER_CTX_set_padding(ctx, 0)

            out_buf = ctypes.create_string_buffer(in_len + 16)
            outlen = ctypes.c_int(0)

            if encrypt_mode:
                rc = _libcrypto.EVP_EncryptUpdate(
                    ctx, out_buf, ctypes.byref(outlen), data, in_len)
                if rc != 1:
                    raise RuntimeError("EVP_EncryptUpdate failed")
            else:
                rc = _libcrypto.EVP_DecryptUpdate(
                    ctx, out_buf, ctypes.byref(outlen), data, in_len)
                if rc != 1:
                    raise RuntimeError("EVP_DecryptUpdate failed")

            ofs = outlen.value
            tail = (ctypes.c_char * 16).from_buffer(out_buf, ofs)
            finallen = ctypes.c_int(0)

            if encrypt_mode:
                rc = _libcrypto.EVP_EncryptFinal_ex(
                    ctx, tail, ctypes.byref(finallen))
                if rc != 1:
                    raise RuntimeError("EVP_EncryptFinal_ex failed")
            else:
                rc = _libcrypto.EVP_DecryptFinal_ex(
                    ctx, tail, ctypes.byref(finallen))
                if rc != 1:
                    raise RuntimeError("EVP_DecryptFinal_ex failed")

            return out_buf.raw[: outlen.value + finallen.value]
        finally:
            if _USE_LEGACY_CTX:
                if hasattr(_libcrypto, 'EVP_CIPHER_CTX_cleanup'):
                    _libcrypto.EVP_CIPHER_CTX_cleanup(ctx)
            else:
                _libcrypto.EVP_CIPHER_CTX_free(ctx)



import unittest

from test import test_support as support

# For scope testing.
g = "Global variable"


class DictComprehensionTest(unittest.TestCase):

    def test_basics(self):
        expected = {0: 10, 1: 11, 2: 12, 3: 13, 4: 14, 5: 15, 6: 16, 7: 17,
                    8: 18, 9: 19}
        actual = {k: k + 10 for k in range(10)}
        self.assertEqual(actual, expected)

        expected = {0: 0, 1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 7, 8: 8, 9: 9}
        actual = {k: v for k in range(10) for v in range(10) if k == v}
        self.assertEqual(actual, expected)

    def test_scope_isolation(self):
        k = "Local Variable"

        expected = {0: None, 1: None, 2: None, 3: None, 4: None, 5: None,
                    6: None, 7: None, 8: None, 9: None}
        actual = {k: None for k in range(10)}
        self.assertEqual(actual, expected)
        self.assertEqual(k, "Local Variable")

        expected = {9: 1, 18: 2, 19: 2, 27: 3, 28: 3, 29: 3, 36: 4, 37: 4,
                    38: 4, 39: 4, 45: 5, 46: 5, 47: 5, 48: 5, 49: 5, 54: 6,
                    55: 6, 56: 6, 57: 6, 58: 6, 59: 6, 63: 7, 64: 7, 65: 7,
                    66: 7, 67: 7, 68: 7, 69: 7, 72: 8, 73: 8, 74: 8, 75: 8,
                    76: 8, 77: 8, 78: 8, 79: 8, 81: 9, 82: 9, 83: 9, 84: 9,
                    85: 9, 86: 9, 87: 9, 88: 9, 89: 9}
        actual = {k: v for v in range(10) for k in range(v * 9, v * 10)}
        self.assertEqual(k, "Local Variable")
        self.assertEqual(actual, expected)

    def test_scope_isolation_from_global(self):
        expected = {0: None, 1: None, 2: None, 3: None, 4: None, 5: None,
                    6: None, 7: None, 8: None, 9: None}
        actual = {g: None for g in range(10)}
        self.assertEqual(actual, expected)
        self.assertEqual(g, "Global variable")

        expected = {9: 1, 18: 2, 19: 2, 27: 3, 28: 3, 29: 3, 36: 4, 37: 4,
                    38: 4, 39: 4, 45: 5, 46: 5, 47: 5, 48: 5, 49: 5, 54: 6,
                    55: 6, 56: 6, 57: 6, 58: 6, 59: 6, 63: 7, 64: 7, 65: 7,
                    66: 7, 67: 7, 68: 7, 69: 7, 72: 8, 73: 8, 74: 8, 75: 8,
                    76: 8, 77: 8, 78: 8, 79: 8, 81: 9, 82: 9, 83: 9, 84: 9,
                    85: 9, 86: 9, 87: 9, 88: 9, 89: 9}
        actual = {g: v for v in range(10) for g in range(v * 9, v * 10)}
        self.assertEqual(g, "Global variable")
        self.assertEqual(actual, expected)

    def test_global_visibility(self):
        expected = {0: 'Global variable', 1: 'Global variable',
                    2: 'Global variable', 3: 'Global variable',
                    4: 'Global variable', 5: 'Global variable',
                    6: 'Global variable', 7: 'Global variable',
                    8: 'Global variable', 9: 'Global variable'}
        actual = {k: g for k in range(10)}
        self.assertEqual(actual, expected)

    def test_local_visibility(self):
        v = "Local variable"
        expected = {0: 'Local variable', 1: 'Local variable',
                    2: 'Local variable', 3: 'Local variable',
                    4: 'Local variable', 5: 'Local variable',
                    6: 'Local variable', 7: 'Local variable',
                    8: 'Local variable', 9: 'Local variable'}
        actual = {k: v for k in range(10)}
        self.assertEqual(actual, expected)
        self.assertEqual(v, "Local variable")

    def test_illegal_assignment(self):
        with self.assertRaisesRegexp(SyntaxError, "can't assign"):
            compile("{x: y for y, x in ((1, 2), (3, 4))} = 5", "<test>",
                    "exec")

        with self.assertRaisesRegexp(SyntaxError, "can't assign"):
            compile("{x: y for y, x in ((1, 2), (3, 4))} += 5", "<test>",
                    "exec")


def test_main():
    support.run_unittest(__name__)

if __name__ == "__main__":
    test_main()



from whatap.pack.pack import Pack
from whatap.pack.pack_enum import PackEnum

class TextRec :
    def __init__(self, div = None, hash = None, text = None ):
        self.div= div
        self.hash= hash
        self.text= text

class TextPack(Pack):
    def __init__(self):
        self.records= []

    def getPackType(self):
	    return PackEnum.PACK_TEXT

    def write(self, dout ):
        super(TextPack, self).write(dout)
        dout.writeDecimal(len(self.records))
        for r in self.records:
            dout.writeByte(r.div)
            dout.writeInt(r.hash)
            dout.writeText(r.text)

    def read(self, din ):
        super(TextPack, self).read(din)
        size = din.readDecimal()
        for i in range(size):
            div = din.readByte()
            hash = din.readInt()
            text = din.readText()
            self.records.append(TextRec(div = div, hash = hash, text = text))

    def addTexts(self, texts =[]):
        self.records += texts



#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# See the License for the specific language governing permissions
# and limitations under the License.
#
#
# Copyright 2011 Grigale Ltd. All rigths reserved.
# Use is sujbect to license terms.
#

import ctypes as C
import struct
import libkstat


class Sysinfo(C.Structure):
    _fields_ = [
            ("updates", C.c_uint),  # uint_t updates
            ("runque", C.c_uint),   # uint_t runque
            ("runocc", C.c_uint),   # uint_t runocc
            ("swpque", C.c_uint),   # uint_t swpque
            ("swpocc", C.c_uint),   # uint_t swpocc
            ("waiting", C.c_uint)   # uint_t waiting
            ]


class Kstat():
    def __init__(self, module='', instance=-1, name=''):
        self._ctl = libkstat.kstat_open()
        self._module = module
        self._inst = instance
        self._name = name

    def __del__(self):
        if libkstat is not None:
            try:
                libkstat.kstat_close(self._ctl)
            except AttributeError:
                pass

    def __str__(self):
        s = 'Module: {0}, instance: {1}, name: {2}'.format(self._module, self._inst, self._name) 
        return s

    def __repr__(self):
        s = 'Kstat("{0}", {1}, "{2}")'.format(self._module, self._inst, self._name) 
        return s

    def lookup(self):
        libkstat.kstat_lookup(self._ctl, self._module, self._inst, self._name)

    def __getitem__(self, triplet):
        module, instance, name = triplet
        ksp = libkstat.kstat_lookup(self._ctl, module, instance, name)
        if not ksp:
            raise KeyError(triplet)
        libkstat.kstat_read(self._ctl, ksp, None)
        ks = ksp.contents
        if ks.ks_type == libkstat.KSTAT_TYPE_RAW:
            value = dict()
            if name == "sysinfo":
                sysinfo = C.cast(ks.ks_data, C.POINTER(Sysinfo)).contents
                value["updates"] = sysinfo.updates
                value["runque"] = sysinfo.runque
                value["runocc"] = sysinfo.runocc
                value["swpque"] = sysinfo.swpque
                value["swpocc"] = sysinfo.swpocc
                value["waiting"] = sysinfo.waiting

        elif ks.ks_type == libkstat.KSTAT_TYPE_NAMED:
            value = dict()
            datap = C.cast(ks.ks_data, C.POINTER(libkstat.kstat_named))
            for i in range(ks.ks_ndata):
                if datap[i].data_type == libkstat.KSTAT_DATA_CHAR:
                    value[datap[i].name] = datap[i].value.c
                elif datap[i].data_type == libkstat.KSTAT_DATA_INT32:
                    value[datap[i].name] = datap[i].value.i32
                elif datap[i].data_type == libkstat.KSTAT_DATA_UINT32:
                    value[datap[i].name] = datap[i].value.ui32
                elif datap[i].data_type == libkstat.KSTAT_DATA_INT64:
                    value[datap[i].name] = datap[i].value.i64
                elif datap[i].data_type == libkstat.KSTAT_DATA_UINT64:
                    value[datap[i].name] = datap[i].value.ui64
                #print datap.contents
                #print dir(datap[i].value)
                #value[datap[i].name] = 0
        elif ks.ks_type == libkstat.KSTAT_TYPE_INTR:
            pass
        elif ks.ks_type == libkstat.KSTAT_TYPE_IO:
            pass
        elif ks.ks_type == libkstat.KSTAT_TYPE_TIMER:
            pass
        else:
            pass

        return value

    def dump(self):
        kc = self._ctl.contents
        ksp = kc.kc_chain
        while ksp:
            ks = ksp.contents
            print ks.ks_module, ks.ks_instance, ks.ks_name, libkstat.kstat_type_names[ks.ks_type], ks.ks_class, ks.ks_ndata, ks.ks_data_size
            ksp = ks.ks_next
        pass 


class KstatValue():
    pass


class NamedKstat(KstatValue):
    def __init__(self):
        pass


def main():
    import pprint as pp
    k = Kstat()
    pp.pprint(k)
    #k.dump()
    pp.pprint(k['unix', 0, 'kstat_types'])
    #pp.pprint(k['unix', 0, 'zio_data_buf2560'])
    pp.pprint(k['audiohd', 0, 'engine_0'])
    #k.lookup()


if __name__ == '__main__':
    main()



from whatap.io.data_outputx import DataOutputX
from whatap.io.data_inputx import DataInputX
from whatap.pack.pack import Pack
from whatap.pack.pack_enum import PackEnum

EVENTSOURCE_FILE       = 1
EVENTSOURCE_WINEVENT   = 2
EVENTSOURCE_SCRIPT     = 3
EVENTSOURCE_AUDIT      = 4
EVENTSOURCE_FILE_WATCH = 5


class SMLogEvent :
	def __init__(self):
		self.eventSource = None
		self.severity = None
		self.filePath = None
		self.logContent = None

		self.winLogFile = None
		self.winType = 0
		self.winSourceName = None
		self.winEventCode = 0
		self.winCreateTime = 0

		self.keyword = None
		self.logRule = None

		# File check fields (for EVENTSOURCE_FILE_WATCH)
		self.checkType = 0
		self.previous = None
		self.current = None
		self.elapsed = 0

	def write(self, out):
		dout = DataOutputX()
		dout.writeByte(self.eventSource)

		dout.writeByte(self.severity)
		if self.filePath:
			dout.writeText( self.filePath)
		else:
			dout.writeText("")

		if self.logContent:
			dout.writeText( self.logContent )
		else:
			dout.writeText("")

		if self.winLogFile :
			dout.writeText( self.winLogFile)
		else:
			dout.writeText("")

		dout.writeInt(self.winType)
		if self.winSourceName :
			dout.writeText( self.winSourceName)
		else :
			dout.writeText("")


		dout.writeInt(self.winEventCode)
		dout.writeLong(self.winCreateTime)

		if self.keyword:
			dout.writeText( self.keyword)
		else:
			dout.writeText("")

		if self.logRule:
			dout.writeText( self.logRule)
		else:
			dout.writeText("")

		# File check fields
		dout.writeByte(self.checkType if self.checkType else 0)
		if self.previous:
			dout.writeText(self.previous)
		else:
			dout.writeText("")
		if self.current:
			dout.writeText(self.current)
		else:
			dout.writeText("")
		dout.writeInt(self.elapsed if self.elapsed else 0)

		out.writeBlob(dout.toByteArray())

	def read(self, inx):
		din = DataInputX( inx.readBlob())

		self.eventSource = din.readByte()
		self.severity = din.readByte()
		filepath = din.readText()
		self.filePath = filepath
		logcontent = din.readText()
		self.logContent = logcontent

		winlogfile = din.readText()
		self.winLogFile = winlogfile
		self.winType = din.readInt()
		winsourcename = din.readText()
		self.winSourceName = winsourcename
		self.winEventCode = din.readInt()
		self.winCreateTime = din.readLong()

		keyword = din.readText()
		self.keyword = keyword

		logRule = din.readText()
		self.logRule = logRule

		# File check fields
		self.checkType = din.readByte()
		self.previous = din.readText()
		self.current = din.readText()
		self.elapsed = din.readInt()


class SMLogEventPack(Pack):
	def __init__(self):
		self.logEvent = []

	def getPackType(self):
		return PackEnum.PACK_SM_LOG_EVENT

	def write(self, dout):
		super(SMLogEventPack, self).write(dout)

		dout.writeDecimal(len(self.logEvent))
		for logEvent in self.logEvent:
			logEvent.write(dout)

	def read(self, din):
		super(SMLogEventPack, self).read(din)
		eventCount = din.readDecimal()
		if eventCount:
			for i in range(eventCount):
				logevent = SMLogEvent()
				self.logEvent.append(logevent.read(din))



import subprocess, shlex, os, time
from StringIO import StringIO
import whatap.util.file_util as file_util
from whatap.util.ip_util import IPUtil as ip_util
from whatap.io.data_outputx import DataOutputX
import whatap.util.process_util as process_util
import whatap.util.logging_util as logging_util
import whatap.util.thread_util as thread_util
import whatap.agent.conf.configure as conf
import psutil
import threading
import multiprocessing
import copy
import procgroup_rule as procgroup_rule

cpucore = float(multiprocessing.cpu_count())


class ProcessPerfInfo:
    def __init__(self):
        self.ppid = None
        self.pid = None
        self.cpu = None
        self.cpuUser = 0.0
        self.cpuSys = 0.0
        self.memoryBytes = None
        self.memoryPercent = None
        self.vsz = 0
        self.readBps = 0.0
        self.writeBps = 0.0
        self.cmd1 = None
        self.cmd2 = None
        self.readIops = 0.0
        self.writeIops = 0.0
        self.user = None
        self.state = None
        self.name = None
        self.createTime = None
        self.net = []
        self.file = []
        self.sharedMemory = 0
        self.threadCount = 0
        self.processCount = 0
        self.totalCharIO = 0
        self.pids = []

    def __str__(self):
        b = StringIO()
        b.write('ppid:')
        b.write(self.ppid)
        b.write('\npid')
        b.write(self.pid)
        b.write('\ncpu')
        b.write(self.cpu)
        b.write('\nmemory')
        b.write(self.memoryBytes)
        b.write('\nsharedMemory')
        b.write(self.sharedMemory)


        return b.getvalue()


class IpPort:
    def __init__(self, localip=0, localport=0, conn=0, ilocaladdr=0, inode=0):
        self.localip = localip
        self.localport = localport
        self.ilocaladdr = ilocaladdr
        self.iremoteaddr = 0
        self.conn = conn
        self.inode = inode

    def __str__(self):
        buf = StringIO()
        buf.write("localip=")
        buf.write(self.localip)
        buf.write("\nlocalport=")
        buf.write(self.localport)
        buf.write("\nilocaladdr=")
        buf.write(self.ilocaladdr)
        buf.write("\niremoteaddr=")
        buf.write(self.iremoteaddr)
        buf.write("\nconn=")
        buf.write(self.conn)
        buf.write("\ninode=")
        buf.write(self.inode)

        return buf.getvalue()


class FileInfo:
    def __init__(self, name=None, size=0):
        self.name = name
        self.size = size

def populateNetworkStatus(processPerfs):
    localips = []
    ipinfo = process_util.executeCmd("ip addr show")

    for line in ipinfo.split('\n'):
        words = line.split()

        if words and words[0] == 'inet':
            localips.append(words[1].split('/')[0])
    inodeportlookup = {}
    ipinodelookup = {}
    remoteconns = []
    for tcpfilepath in ("/proc/net/tcp", "/proc/net/tcp6"):
        tcpContent = file_util.readContent(tcpfilepath)
        if tcpContent:
            i = 0
            for line in tcpContent.split("\n"):
                i += 1
                if i == 1 or not line:
                    continue

                words = line.split()
                inode = int(words[9])
                localIPBytes = ip_util.parseHexString(words[1])
                if len(localIPBytes) != 6:
                    continue
                localIP = DataOutputX.toInt(localIPBytes[:4])
                localPort = DataOutputX.toShort(localIPBytes[4:])
                localIPPortInt = DataOutputX.toLong6(localIPBytes)
                remoteIPBytes = ip_util.parseHexString(words[2])
                if len(remoteIPBytes) != 6:
                    continue
                remoteIPInt = DataOutputX.toInt(remoteIPBytes[:4])
                remotePortInt = DataOutputX.toShort(remoteIPBytes[4:])
                ipport = IpPort(localip=localIP, localport=localPort, conn=0, ilocaladdr=localIPPortInt,
                                inode=int(inode))
                if remoteIPInt == 0 and remotePortInt == 0:
                    inodeportlookup[inode] = ipport
                    if localIPPortInt > 0:
                        ipinodelookup[localIPPortInt] = ipport
                        for ip in localips:
                            ipbytes = ip_util.toBytes(ip)
                            deviceIPPortBytes = StringIO()
                            deviceIPPortBytes.write(ipbytes)
                            deviceIPPortBytes.write(localIPBytes[4:])
                            deviceIPPortInt = DataOutputX.toLong6(deviceIPPortBytes.getvalue())
                            ipinodelookup[deviceIPPortInt] = ipport

                else:
                    remoteconns.append(ipport)

    for pipport in remoteconns:
        if pipport.ilocaladdr in ipinodelookup:
            ipinodelookup[pipport.ilocaladdr].conn += 1

    # print 'ipinodelookup'
    # from pprint import pprint
    # for k, v in ipinodelookup.items():
    #    print k,v

    for processPerf in processPerfs:
        ports = []
        filesinfo = []
        searchDir = os.path.join("/proc", str(processPerf.pid), "fd")

        if not os.path.exists(searchDir):
            continue
        for filename in os.listdir(searchDir):
            path = os.path.join(searchDir, filename)

            if os.path.islink(path):
                link = os.readlink(path)

                if link.startswith("socket:["):
                    inode = int(link[8:len(link) - 1])
                    # print 'inode', inode, inode in inodeportlookup
                    if inode in inodeportlookup:
                        ipport = inodeportlookup[inode]
                        ports.append(IpPort(localip=ipport.localip, localport=ipport.localport, conn=ipport.conn))

                elif len(link) > 0 and not link.startswith("/dev") and link[0] == '/':
                    try:
                        filestat = os.stat(link)
                        # print 'file ',processPerf.cmd1, link, filestat.st_size
                        if filestat and link.endswith(".log"):
                            fileinfo = FileInfo(name=link, size=filestat.st_size)
                            filesinfo.append(fileinfo)
                            # print processPerf.cmd1, link
                    except Exception, e:
                        pass
        processPerf.net = ports
        processPerf.file = filesinfo

class _ProcPerfCache:
    def __init__(self, cpu_times=None, proc_map=None, timestamp=None):
        self.cpu_times = cpu_times
        self.proc_map = proc_map
        if timestamp:
            self.timestamp = timestamp
        else:
            self.timestamp = time.time()


jiff = 100

def parseNetwork(conns):
    port_map = {}
    ports= []
    for c in conns:
        if c.family == 2 and c.type == 2 and c.status=='LISTEN':
            ipport = IpPort(localip=ip_util.strToInt(c.laddr.ip), localport=c.laddr.port, conn=0)
            port_map[c.laddr.port] = ipport
            ports.append(ipport)
    for c in conns:
        if c.family == 2 and c.type == 2 and c.status == 'ESTABLISHED' and c.laddr.port in port_map:
            port_map[c.laddr.port].conn += 1
    return ports

def parseOpenFile(files):
    flookup = {}
    fileusage =[]
    for f in files:
        if f.path and f.path.endswith('.log'):
            flookup[f.path] = 0
    for p in flookup.keys():
        filestat = os.stat(p)
        if filestat:
            fileusage.append(FileInfo(name=p, size=filestat.st_size))

    return fileusage



proc_shm_cache = {}
data_map_lock = threading.Lock()
update_condition = threading.Condition()
update_queue = []

def updateShmCache():
    while True:
        with update_condition:
            while not update_queue:
                update_condition.wait()

            proc = update_queue.pop(0)

        with data_map_lock:
            sharedMemory = 0
            try:
                if proc.pid not in proc_shm_cache or proc.create_time() != proc_shm_cache[proc.pid][1]:
                    for m in proc.memory_maps():
                        if m.path == '[shmid]':
                            sharedMemory += m.rss
                proc_shm_cache[proc.pid] = (sharedMemory, proc.create_time())
            except psutil.ZombieProcess, e:
                logging_util.debug("zombie process skip", e)
                continue
            except psutil.NoSuchProcess, e: 
                logging_util.debug("process cpu time error : ", e)
                continue


def setSharedMemory(proc):
    if data_map_lock.acquire(False):
        try:
            if proc.pid in proc_shm_cache and proc.create_time() == proc_shm_cache[proc.pid][1]:
                (sharedMemory, _) = proc_shm_cache[proc.pid]
                return sharedMemory
            else:
                with update_condition:
                    update_queue.append(proc)
                    update_condition.notify()
                return 0

        finally:
            data_map_lock.release()
    else:
        with update_condition:
            update_queue.append(proc)
            update_condition.notify()
        return 0


oldProcPerfCache = None



PROC_STATUSES = {
    psutil._pssunos.cext.SSLEEP: psutil._common.STATUS_SLEEPING,
    psutil._pssunos.cext.SRUN: psutil._common.STATUS_RUNNING,
    psutil._pssunos.cext.SZOMB: psutil._common.STATUS_ZOMBIE,
    psutil._pssunos.cext.SSTOP: psutil._common.STATUS_STOPPED,
    psutil._pssunos.cext.SIDL: psutil._common.STATUS_IDLE,
    psutil._pssunos.cext.SONPROC: psutil._common.STATUS_RUNNING,  # same as run
    psutil._pssunos.cext.SWAIT: psutil._common.STATUS_WAITING,
}


def getProcessPerfList():
    global proc_shm_cache
    proc_map = {}

    vmem = psutil.virtual_memory()
    totalMem = vmem.total
    proc_list = []

    cpu_times = psutil.cpu_times(percpu=False)
    cpu_jiff_diff = 0

    global oldProcPerfCache
    if oldProcPerfCache:
        all_cpu = time.time()
        last_all_cpu = oldProcPerfCache.timestamp
        cpu_jiff_diff = float(all_cpu - last_all_cpu) * cpucore

    try:
        procInfo = psutil.proc_detail_info()
        #0 pid
        #1 ppid
        #2 name
        #3 cmdline
        #4 exe
        #5 username
        #6 rss
        #7 vms
        #8 createtime
        #9 thread count
        #10 status code
        #11 utime
        #12 stime
        #13 ioch
        #14 inblk
        #15 oublk
        for proc in procInfo.values():
            now_proc_times = proc[11] + proc[12]
            pid = proc[0]
            proc_map[pid] = now_proc_times

            if not oldProcPerfCache:
                continue

            if pid not in oldProcPerfCache.proc_map:
                continue

            p = ProcessPerfInfo()
            p.pid = pid
            p.ppid = proc[1]
            p.user = proc[5]
            
            old_proc_times = oldProcPerfCache.proc_map[pid]
            if cpu_jiff_diff != 0:
                p.cpu = float(100) * float(now_proc_times - old_proc_times) / cpu_jiff_diff

            p.memoryBytes = proc[6]
            p.memoryPercent = 100.0 * p.memoryBytes / totalMem
            p.cmd1 = proc[4]
            p.cmd2 = proc[3]
            p.createTime = int(proc[8])

            p.state = PROC_STATUSES.get(proc[10], '?')

            #TODO 
            if conf.GetConfig().serverProcessConnectCheck:
                p.net = parseNetwork(psutil._pssunos.net_connections('inet', pid))
            #TODO 
            if conf.GetConfig().serverProcessFDCheck:
                processObj = psutil.Process(pid)
                p.file = parseOpenFile(processObj.open_files())

            proc_list.append(p)
    except Exception as e:
        logging_util.error("proc(perfList) detail error : ", e)
        return None

    procPerf = _ProcPerfCache(cpu_times = cpu_times, proc_map = proc_map)
    oldProcPerfCache = procPerf
    return proc_list

init=False

def GetProc():
    global init
    if init:
        thread_util.async(updateShmCache)
        init=False

    logging_util.debug("Get Proc Start")
    procPerfs = getProcessPerfList()
    if procPerfs == None:
        logging_util.debug("Get Proc End : None")
    else:
        logging_util.debug("Get Proc End : ", len(procPerfs))

    return procPerfs




def GetTotalProc():
    try:
        proc, thread, defunct = psutil.proc_total_info()
        return (proc, thread, defunct)
    except:
        return (None, None, None)


oldGroupProcPerfCache = None

def GetTopNAndGroupProc(n):
    cpu_threshold = 0.1
    rss_threshold = 20 * 1024 * 1024

    cpu_topn = n / 4 * 3
    memory_topn = n / 4 

    proc_map = {}
    proc_list = []

    cpu_times = psutil.cpu_times(percpu=False)
    cpu_jiff_diff = 0
    timediff = 0
    now = time.time()
    global oldGroupProcPerfCache 
    if oldGroupProcPerfCache:
        oldTime = oldGroupProcPerfCache.timestamp
        timediff = float(now - oldTime)
        cpu_jiff_diff = timediff * cpucore


    try:
        procInfo = psutil.proc_detail_info()

        #0 pid
        #1 ppid
        #2 name
        #3 cmdline
        #4 exe
        #5 username
        #6 rss
        #7 vms
        #8 createtime
        #9 thread count
        #10 status code
        #11 utime
        #12 stime
        #13 ioch
        #14 inblk
        #15 oublk
        for proc in procInfo.values():
            now_utime = proc[11]
            now_stime = proc[12]
            now_proc_times = now_utime + now_stime
            total_char_io = proc[13]
            pid = proc[0]

            proc_map[pid] = (now_utime, now_stime, total_char_io)

            if not oldGroupProcPerfCache:
                continue

            if pid not in oldGroupProcPerfCache.proc_map:
                continue

            p = ProcessPerfInfo()
            p.pid = pid
            p.ppid = proc[1]
            p.user = proc[5]

            old_entry = oldGroupProcPerfCache.proc_map[pid]
            old_utime = old_entry[0]
            old_stime = old_entry[1]
            old_total_char_io = old_entry[2] if len(old_entry) > 2 else old_entry[1]
            old_proc_times = old_utime + old_stime

            if cpu_jiff_diff != 0:
                p.cpu = float(100) * float(now_proc_times - old_proc_times) / cpu_jiff_diff
                p.cpuUser = float(100) * float(now_utime - old_utime) / cpu_jiff_diff
                p.cpuSys = float(100) * float(now_stime - old_stime) / cpu_jiff_diff
            else:
                p.cpu = 0.0
                p.cpuUser = 0.0
                p.cpuSys = 0.0

            p.totalCharIO = float(total_char_io - old_total_char_io) / float(timediff)
            p.memoryBytes = proc[6]
            p.vsz = proc[7]
            p.cmd2 = proc[3]
            p.cmd1 = proc[3].split(' ')[0] if proc[3] else proc[2]
            p.createTime = int(proc[8])
            p.threadCount = proc[9]
            p.state = PROC_STATUSES.get(proc[10], '?')
            p.processCount = 1
            p.pids = [pid]

            proc_list.append(p)
    except Exception as e:
        logging_util.error("proc(TopN_Group) detail error : ", e)
        return None, None

    procPerf = _ProcPerfCache(cpu_times = cpu_times, proc_map = proc_map, timestamp = now)
    oldGroupProcPerfCache = procPerf
    procgroup_rule.gc_pid_cache(set(proc_map.keys()))
    cpu_top = sorted([proc for proc in proc_list if proc.cpu > cpu_threshold], key = lambda p: p.cpu, reverse = True)[:cpu_topn]
    cpu_top_pids = {proc.pid for proc in cpu_top}

    rss_top = sorted([proc for proc in proc_list if proc.pid not in cpu_top_pids and proc.memoryBytes > rss_threshold], key = lambda p: p.memoryBytes, reverse = True)[:memory_topn]

    topn = cpu_top + rss_top

    process_group = {}

    for proc in proc_list:
        if proc.state == 'zombie':
            key = ('defunct', proc.user)
            group_name = 'defunct'
        else:
            matched = procgroup_rule.find_custom_group(proc.pid, proc.user, proc.cmd2)
            if matched:
                key = (proc.user, matched)
                group_name = matched
            else:
                key = (proc.cmd1, proc.user)
                group_name = None

        if key not in process_group:
            p = copy.deepcopy(proc)
            p.name = group_name if group_name else proc.cmd1
            process_group[key] = p
        else:
            process_group[key].cpu += proc.cpu
            process_group[key].cpuUser += proc.cpuUser
            process_group[key].cpuSys += proc.cpuSys
            process_group[key].memoryBytes += proc.memoryBytes
            process_group[key].vsz += proc.vsz
            process_group[key].threadCount += proc.threadCount
            process_group[key].processCount += 1
            process_group[key].totalCharIO += proc.totalCharIO
            process_group[key].pids.extend(proc.pids)

    return (topn, process_group)


def test():
    while True:
        procs = GetProc()
        print
        procs
        if not procs:
            time.sleep(5)
            continue
        time.sleep(5)


if __name__ == '__main__':
    test()



import unittest
import idlelib.RstripExtension as rs
from idlelib.idle_test.mock_idle import Editor

class rstripTest(unittest.TestCase):

    def test_rstrip_line(self):
        editor = Editor()
        text = editor.text
        do_rstrip = rs.RstripExtension(editor).do_rstrip

        do_rstrip()
        self.assertEqual(text.get('1.0', 'insert'), '')
        text.insert('1.0', '     ')
        do_rstrip()
        self.assertEqual(text.get('1.0', 'insert'), '')
        text.insert('1.0', '     \n')
        do_rstrip()
        self.assertEqual(text.get('1.0', 'insert'), '\n')

    def test_rstrip_multiple(self):
        editor = Editor()
        #  Uncomment following to verify that test passes with real widgets.
##        from idlelib.EditorWindow import EditorWindow as Editor
##        from tkinter import Tk
##        editor = Editor(root=Tk())
        text = editor.text
        do_rstrip = rs.RstripExtension(editor).do_rstrip

        original = (
            "Line with an ending tab    \n"
            "Line ending in 5 spaces     \n"
            "Linewithnospaces\n"
            "    indented line\n"
            "    indented line with trailing space \n"
            "    ")
        stripped = (
            "Line with an ending tab\n"
            "Line ending in 5 spaces\n"
            "Linewithnospaces\n"
            "    indented line\n"
            "    indented line with trailing space\n")

        text.insert('1.0', original)
        do_rstrip()
        self.assertEqual(text.get('1.0', 'insert'), stripped)

if __name__ == '__main__':
    unittest.main(verbosity=2, exit=False)



from whatap.value.number_value import NumberValue
from whatap.value.value_enum import ValueEnum


class DecimalValue(NumberValue):
    def __init__(self, value=None):
        super(DecimalValue, self).__init__()
        self.value = value

    def compareTo(self, object):
        if isinstance(object, DecimalValue):
            if self.value < object.value:
                return -1
            elif self.value == object.value:
                return 0
            else:
                return 1
        return 1

    def equals(self, object):
        if isinstance(object, DecimalValue):
            return self.value == object.value
        return False

    def hashCode(self):
        return self.value ^ (self.value >> 32)

    def write(self, dout):
        dout.writeDecimal(self.value)

    def read(self, din):
        self.value = din.readDecimal()
        return self

    def getValueType(self):
        return ValueEnum.DECIMAL

    def toObject(self):
        return self.value


from whatap.value.value import Value
from whatap.value.value_enum import ValueEnum

from whatap.util.compare_util import CompareUtil


class TextValue(Value):
    def __init__(self, value=None):
        super(TextValue, self).__init__()
        if not value:
            self.value = ''
        else:
            self.value = value
    
    def compareTo(self, object):
        if isinstance(object, TextValue):
            return CompareUtil.compareTo(self.value, object.value)
        return 1
    
    def equals(self, object):
        if isinstance(object, TextValue):
            if self.value is None:
                return object.value is None
            return self.value == object.value
        return False
    
    def hashCode(self):
        if not self.value:
            return 0
        
        hash = 0
        if not hash and len(self.value):
            for v in self.value:
                hash = 31 * hash + v
        return hash
    
    def write(self, dout):
        dout.writeText(self.value)
    
    def read(self, din):
        self.value = din.readText()
        return self
    
    def getValueType(self):
        return ValueEnum.TEXT

    def toObject(self):
        return self.value



Configuration = {
    "enabled": ('Enabled',True),
    "license": ('License',None),
    "whatap.server.host": ('WhatapHost', ''),
    "whatap.server.port":  ('WhatapPort', 6600),
    "cypher_level":  ('CypherLevel',128),
    "EncryptLevel": 0,

    "time_sync_interval": ('TimeSyncInterval',30),

    "downcheck.listen.enabled":    ('DownCheckListenEnabled', False),
    "downcheck.enabled":    ('DownCheckEnabled', False),
    "downcheck.interval":   ('DownCheckInterval',0),
    "downcheck.timeout":    ('DownCheckTimeout',0),
    "downcheck.concurrent": ('DownCheckConcurrent',0),
    "downcheck.target":  ('DownCheckTargetStr',""),
    "downcheck.concurrent": ('DownCheckConcurrent', 0),
    "peer.port": ('PeerPort', 10120),

    "createdtime":         ('CreatedTime',0),

    "tag.agent": ('Tags',''),
    "sample_type": ('SampleType',None),

    "execution.cache.timeout": ('ExecutionScriptCacheTimeout', 60000),

    "debug": ('Debug',False),

    "log.line.limit": ('LogLineLimit', 60),
    "file.count.interval": ('FileCountInterval', 60),

    "housekeeping.enabled":('HouseKeepEnabled', True),
    "housekeeping.hour": ('HouseKeepHour', 2),
	"housekeeping.cooltime": ('HouseKeepCooltime', 12),
    "healthcheck.cooltime": ('HealthCheckCoolTime', 3),
    "agentless.interval": ("AgentlessInterval", 60000),
    "agentless.timeout":("AgentlessTimeout", 10000),
    "agentless.concurrent":('AgentlessConcurrent', 5),
    "agentless.items":('AgentlessItems', []),

    "sunos.memstat.enabled":("SunosMdbEnabled", False),
    "sunos.memstat.interval": ("MemstatInterval", 30),
    "sunos.disk.quota.enabled": ("SunosDiskQuotaEnabled", False),
    "aix.diskstat.interval": ("DiskstatInterval", 5),
    "process.fdcheck":('serverProcessFDCheck', False),
    "process.connectcheck":('serverProcessConnectCheck', False),
    "process.moncheck": ("serverProcessMonCheck", True),
    "tcpcheck_timeout": ("tcpcheck_timeout", 4),
    "process_update_interval": ("process_update_interval", 20),
    "oid": ("oid",0),
    "oname": ("oname", ''),

    "task_timeout_except": ("task_timeout_except", True),
    "task_timeout": ("task_timeout", 2),
    "task_health_check_count": ("task_health_check_count", 5),
    "task_disk": ("task_disk", True),
    "disk.io.enabled": ("DiskIOEnabled", True),
    "task_osinfo": ("task_osinfo", True),
    "task_base": ("task_base", True),
    "task_net": ("task_net", True),
    "task_proc": ("task_proc", True),
    "task_config": ("task_config", True),
    "task_filesystem": ("task_filesystem", True),
    "task_tcpcheck": ("task_tcpcheck", True),
    "task_logevent": ("task_logevent", True),

    "hostinfov2.enabled":("HostInfoV2Enabled", False),
    "categoryv1.enabled":("CategoryV1Enabled", True),
    "categoryv2.enabled":("CategoryV2Enabled", False),

    "categoryv2.interval":("CategoryV2Interval", 2),
    "categoryv2.folding.interval":("CategoryV2FoldingInterval", 5),

    "ipreg":("ipreg", False),
    "log.size":("logSize", 4), 
    "log.backup":("logBackup", 3),
    "log.path":("logPath", "/var/log"),

    "aix.errpt.enabled":("ErrptReditectEnabled", True),
    "aix.errpt.Interval":("ErrptReditectInterval", 60),
    "aix.errpt.log.path":("ErrptLogPath", "/var/log/whatap_errpt_message.log"),
    "aix.errpt.log.size":("ErrptLogSize", 4),
    "aix.errpt.log.backup":("ErrptLogFileCount", 3),

    "package.dir":("PackageDir", "/tmp/whatap-package"),

    "cronjob.enabled":("CronjobEnabled", False),
    "cronjob.path":("CronjobPath", "/usr/whatap/infra/cronjob"),
    "cronjob.conf":("CronjobConf", "cronjob.json"),
    "cronjob.timeout":("CronjobTimeout", 5),

}
